from typing import Callable, Iterable, Literal
from . import BaseCrossValidator
from numpy.random import RandomState
from matplotlib.figure import Figure
from ._split import BaseShuffleSplit
from matplotlib.axes import Axes
from matplotlib.artist import Artist
from .._typing import ArrayLike, MatrixLike, Int, Float
from ..utils import check_matplotlib_support as check_matplotlib_support
from . import learning_curve as learning_curve
import numpy as np


class LearningCurveDisplay:
    fill_between_: Artist | None = ...
    lines_: Artist | None = ...
    errorbar_: Artist | None = ...
    figure_: Figure = ...
    ax_: Axes = ...

    def __init__(
        self,
        *,
        train_sizes: ArrayLike,
        train_scores: MatrixLike,
        test_scores: MatrixLike,
        score_name: None | str = None,
    ) -> None:
        ...

    def plot(
        self,
        ax: None | Axes = None,
        *,
        negate_score: bool = False,
        score_name: None | str = None,
        score_type: Literal["test", "train", "both", "test"] = "test",
        log_scale: bool = False,
        std_display_style: None
        | Literal["errorbar", "fill_between", "fill_between"] = "fill_between",
        line_kw: None | dict = None,
        fill_between_kw: None | dict = None,
        errorbar_kw: None | dict = None,
    ) -> LearningCurveDisplay:
        ...

    @classmethod
    def from_estimator(
        cls,
        estimator,
        X: MatrixLike,
        y: None | MatrixLike | ArrayLike,
        *,
        groups: None | ArrayLike = None,
        train_sizes: ArrayLike = ...,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        scoring: None | str | Callable = None,
        exploit_incremental_learning: bool = False,
        n_jobs: None | Int = None,
        pre_dispatch: str | Int = "all",
        verbose: Int = 0,
        shuffle: bool = False,
        random_state: RandomState | None | Int = None,
        error_score: str | Float = ...,
        fit_params: None | dict = None,
        ax: None | Axes = None,
        negate_score: bool = False,
        score_name: None | str = None,
        score_type: Literal["test", "train", "both", "test"] = "test",
        log_scale: bool = False,
        std_display_style: None
        | Literal["errorbar", "fill_between", "fill_between"] = "fill_between",
        line_kw: None | dict = None,
        fill_between_kw: None | dict = None,
        errorbar_kw: None | dict = None,
    ) -> LearningCurveDisplay:
        ...



from typing import Mapping
import importlib
import importlib.util
import os
import sys

def attach(
    package_name: str,
    submodules: set | None = None,
    submod_attrs: Mapping | None = None,
): ...
def load(fullname: str): ...



import os
from collections import OrderedDict
from typing import Any

from transformers.configuration_utils import PretrainedConfig
from transformers.tokenization_utils import PreTrainedTokenizer
from transformers.tokenization_utils_fast import PreTrainedTokenizerFast

from .auto_factory import _LazyAutoMapping

TOKENIZER_MAPPING_NAMES: OrderedDict[str, tuple[str|None, str|None]]
TOKENIZER_MAPPING: _LazyAutoMapping
CONFIG_TO_TYPE: dict[type[PretrainedConfig], str]

def tokenizer_class_from_name(class_name: str) -> PreTrainedTokenizer| PreTrainedTokenizerFast: ...

def get_tokenizer_config(
    pretrained_model_name_or_path: str | os.PathLike[str], 
    cache_dir: str | os.PathLike[str] | None = None, 
    force_download: bool = False,
    resume_download: bool = False, 
    proxies: dict[str, str] | None = None, 
    use_auth_token: bool | str | None = None,
    revision: str | None = None, 
    local_files_only: bool = False,
    subfolder: str = "",
    **kwargs
) -> dict[str, Any]: ...

class AutoTokenizer:
    def __init__(self) -> None: ...
    @classmethod
    def from_pretrained(
        cls,
        pretrained_model_name_or_path: str | os.PathLike[str], 
        *inputs, 
        **kwargs
    ) -> PreTrainedTokenizer | PreTrainedTokenizerFast: ...
    def register(config_class, slow_tokenizer_class = None, fast_tokenizer_class = None) -> None: ...



from numpy.distutils.core import setup
from numpy.distutils.misc_util import Configuration
from ._build_utils import cythonize_extensions
import sys
import os


def configuration(parent_package: str = "", top_path=None):
    ...



from .graph import Graph as Graph
from .digraph import DiGraph as DiGraph
from .multigraph import MultiGraph as MultiGraph
from .multidigraph import MultiDiGraph as MultiDiGraph
from .ordered import *

from .function import (
    nodes as nodes,
    edges as edges,
    degree as degree,
    degree_histogram as degree_histogram,
    neighbors as neighbors,
    number_of_nodes as number_of_nodes,
    number_of_edges as number_of_edges,
    density as density,
    is_directed as is_directed,
    info as info,
    freeze as freeze,
    is_frozen as is_frozen,
    subgraph as subgraph,
    subgraph_view as subgraph_view,
    induced_subgraph as induced_subgraph,
    reverse_view as reverse_view,
    edge_subgraph as edge_subgraph,
    restricted_view as restricted_view,
    to_directed as to_directed,
    to_undirected as to_undirected,
    add_star as add_star,
    add_path as add_path,
    add_cycle as add_cycle,
    create_empty_copy as create_empty_copy,
    set_node_attributes as set_node_attributes,
    get_node_attributes as get_node_attributes,
    set_edge_attributes as set_edge_attributes,
    get_edge_attributes as get_edge_attributes,
    all_neighbors as all_neighbors,
    non_neighbors as non_neighbors,
    non_edges as non_edges,
    common_neighbors as common_neighbors,
    is_weighted as is_weighted,
    is_negatively_weighted as is_negatively_weighted,
    is_empty as is_empty,
    selfloop_edges as selfloop_edges,
    nodes_with_selfloops as nodes_with_selfloops,
    number_of_selfloops as number_of_selfloops,
    path_weight as path_weight,
    is_path as is_path,
)

from . import filters as filters

from . import coreviews as coreviews
from . import graphviews as graphviews
from . import reportviews as reportviews



from typing import Mapping
import importlib
import importlib.util
import inspect
import os
import sys
import types

__all__ = ["attach"]

def attach(
    module_name: str, submodules: set | None = None, submod_attrs: Mapping | None = None
): ...

class DelayedImportErrorModule(types.ModuleType):
    def __init__(self, frame_data, *args, **kwargs): ...
    def __getattr__(self, x): ...



import copy
from collections import defaultdict
from itertools import combinations
from operator import itemgetter

from ...classes.graph import Graph
from ...algorithms.flow import (
    build_residual_network,
    edmonds_karp,
    shortest_augmenting_path,
)

from .utils import build_auxiliary_node_connectivity

default_flow_func = ...

__all__ = ["all_node_cuts"]

def all_node_cuts(G: Graph, k=None, flow_func=None): ...



# FIXME: It would be better for typing to add the exact imports it has
# here instead of "import *"...

from django.contrib.gis.db.models.aggregates import *  # noqa:F403
from django.contrib.gis.db.models.fields import (
    GeometryCollectionField as GeometryCollectionField,
)
from django.contrib.gis.db.models.fields import GeometryField as GeometryField
from django.contrib.gis.db.models.fields import LineStringField as LineStringField
from django.contrib.gis.db.models.fields import (
    MultiLineStringField as MultiLineStringField,
)
from django.contrib.gis.db.models.fields import MultiPointField as MultiPointField
from django.contrib.gis.db.models.fields import MultiPolygonField as MultiPolygonField
from django.contrib.gis.db.models.fields import PointField as PointField
from django.contrib.gis.db.models.fields import PolygonField as PolygonField
from django.contrib.gis.db.models.fields import RasterField as RasterField
from django.db.models import *  # noqa:F403



from .branchings import (
    branching_weight as branching_weight,
    greedy_branching as greedy_branching,
    maximum_branching as maximum_branching,
    minimum_branching as minimum_branching,
    maximum_spanning_arborescence as maximum_spanning_arborescence,
    minimum_spanning_arborescence as minimum_spanning_arborescence,
    ArborescenceIterator as ArborescenceIterator,
    Edmonds as Edmonds,
)
from .coding import (
    from_nested_tuple as from_nested_tuple,
    from_prufer_sequence as from_prufer_sequence,
    NotATree as NotATree,
    to_nested_tuple as to_nested_tuple,
    to_prufer_sequence as to_prufer_sequence,
)
from .mst import (
    minimum_spanning_edges as minimum_spanning_edges,
    maximum_spanning_edges as maximum_spanning_edges,
    minimum_spanning_tree as minimum_spanning_tree,
    maximum_spanning_tree as maximum_spanning_tree,
    random_spanning_tree as random_spanning_tree,
    partition_spanning_tree as partition_spanning_tree,
    EdgePartition as EdgePartition,
    SpanningTreeIterator as SpanningTreeIterator,
)
from .recognition import (
    is_arborescence as is_arborescence,
    is_branching as is_branching,
    is_forest as is_forest,
    is_tree as is_tree,
)
from .operations import join as join
from .decomposition import junction_tree as junction_tree



import numpy as np

from .._shared import utils

def _hog_normalize_block(block, method, eps=1e-5): ...
def _hog_channel_gradient(channel): ...
@utils.channel_as_last_axis(multichannel_output=False)
@utils.deprecate_multichannel_kwarg(multichannel_position=8)
def hog(
    image,
    orientations: int = 9,
    pixels_per_cell=...,
    cells_per_block=...,
    block_norm="L2-Hys",
    visualize: bool = False,
    transform_sqrt: bool = False,
    feature_vector: bool = True,
    multichannel: bool | None = None,
    *,
    channel_axis: int | None = None
): ...



import numpy as np
from typing import Generator, Literal
from ._typing import *
import contextlib
from packaging.version import parse as parse_version
from . import _docstring, _version, cbook, rcsetup
from . import colors as colors

class __getattr__:

    URL_REGEX = ...

def set_loglevel(
    level: Literal["notset", "debug", "info", "warning", "error", "critical"]
)-> None: ...

class ExecutableNotFoundError(FileNotFoundError):
    ...

def checkdep_usetex(s)-> bool: ...
def get_configdir() -> str: ...
def get_cachedir() -> str: ...
def get_data_path() -> str: ...
def matplotlib_fname() -> str: ...

class RcParams(dict):

    validate = ...
    def __init__(self, *args, **kwargs) -> None: ...
    def find_all(self, pattern)-> RcParams: ...
    def copy(self)-> RcParams: ...

def rc_params(fail_on_error: bool = ...) -> RcParams: ...
def is_url(filename: str) -> bool: ...
def rc_params_from_file(
    fname: str | PathLike, fail_on_error: bool = ..., use_default_template: bool = ...
) -> RcParams: ...

rcParamsDefault: RcParams = ...
rcParams: RcParams = ...
rcParamsOrig: RcParams = ...

def rc(group, **kwargs)-> None: ...
def rcdefaults()-> None: ...
def rc_file_defaults()-> None: ...
def rc_file(fname: str | PathLike, *, use_default_template: bool = ...)-> None: ...
@contextlib.contextmanager
def rc_context(rc: dict = ..., fname: str | PathLike = ...)-> Generator: ...
def use(backend: str, *, force: bool = True)-> None: ...
def get_backend() -> str: ...
def interactive(b) -> bool: ...
def is_interactive() -> bool: ...

default_test_modules: list[str] = ...

def test(verbosity=..., coverage=..., **kwargs): int: ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from __future__ import division

import builtins
import os
import sys

from . import backends
from .backends import CORE_BACKENDS, BACKEND_NAMES, BACKENDMAP, TRIED_BACKENDS
from .. import config
from .base import BaseApplicationBackend as ApplicationBackend  # noqa
from ._detect_eventloop import _get_running_interactive_framework
from ..util import logger

class Application(object):
    def __init__(self, backend_name: str | None = None): ...
    def __repr__(self): ...
    @property
    def backend_name(self): ...
    @property
    def backend_module(self): ...
    def process_events(self): ...
    def sleep(self, duration_sec: float): ...
    def create(self): ...
    def is_interactive(self): ...
    def is_notebook(self): ...
    def run(self, allow_interactive: bool = True): ...
    def reuse(self): ...
    def quit(self): ...
    @property
    def native(self): ...
    def _use(self, backend_name=None): ...



import warnings

from ..classes.graph import Graph

__all__ = ["read_shp", "write_shp"]

def read_shp(
    path, simplify: bool = True, geom_attrs: bool = True, strict: bool = True
): ...
def edges_from_line(geom, attrs, simplify=True, geom_attrs=True): ...
def write_shp(G: Graph, outdir): ...



# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------

import os
import re
from subprocess import CalledProcessError

from ..logs import logger
from ..wrappers import run_subprocess

def _get_dpi_from(cmd, pattern, func): ...
def _xrandr_calc(x_px, y_px, x_mm, y_mm): ...
def get_dpi(raise_error: bool = True) -> float: ...



from typing import Callable, ClassVar, TypeVar
from .base import BaseEstimator, RegressorMixin, MultiOutputMixin
from .utils.validation import check_is_fitted as check_is_fitted
from numpy import ndarray
from numbers import Integral as Integral, Real as Real
from .utils._param_validation import Interval as Interval, StrOptions as StrOptions
from scipy.sparse import spmatrix
from .metrics.pairwise import (
    PAIRWISE_KERNEL_FUNCTIONS as PAIRWISE_KERNEL_FUNCTIONS,
    pairwise_kernels as pairwise_kernels,
)
from ._typing import Float, ArrayLike, Int, MatrixLike

KernelRidge_Self = TypeVar("KernelRidge_Self", bound="KernelRidge")


import numpy as np


class KernelRidge(MultiOutputMixin, RegressorMixin, BaseEstimator):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    X_fit_: ndarray | spmatrix = ...
    dual_coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        alpha: ArrayLike | Float = 1,
        *,
        kernel: str | Callable = "linear",
        gamma: None | Float = None,
        degree: Int = 3,
        coef0: Float = 1,
        kernel_params: None | dict = None,
    ) -> None:
        ...

    def fit(
        self: KernelRidge_Self,
        X: MatrixLike | ArrayLike,
        y: MatrixLike | ArrayLike,
        sample_weight: None | ArrayLike = None,
    ) -> KernelRidge_Self:
        ...

    def predict(self, X: MatrixLike | ArrayLike) -> ndarray:
        ...



from __future__ import division

import numpy as np
from math import gcd

class TorusKnot(object):
    def __init__(
        self,
        p: int = 3,
        q: int = 2,
        num_points: int = 100,
        major_radius: float = 10.0,
        minor_radius: float = 5.0,
    ): ...
    def _calculate_vertices(self): ...
    @property
    def first_component(self): ...
    @property
    def components(self): ...
    @property
    def num_components(self): ...
    @property
    def q(self): ...
    @q.setter
    def q(self, q): ...
    @property
    def p(self): ...
    @p.setter
    def p(self, p): ...
    @property
    def minor_radius(self): ...
    @minor_radius.setter
    def minor_radius(self, r): ...
    @property
    def major_radius(self): ...
    @major_radius.setter
    def major_radius(self, r): ...
    @property
    def num_points(self): ...
    @num_points.setter
    def num_points(self, r): ...



from ..classes import DiGraph, MultiDiGraph
from ..utils import not_implemented_for
from ..classes.graph import Graph

__all__ = ["stochastic_graph"]

def stochastic_graph(G: Graph, copy: bool = True, weight="weight"): ...



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy.sparse.linalg.eigen.arpack._arpack, version: $Revision: $
import typing
import builtins as _mod_builtins

__doc__: str
__file__: str
__name__: str
__package__: str
__version__: bytes
def cnaupd(ido, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, rwork, info, n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "ido,tol,resid,v,iparam,ipntr,info = cnaupd(ido,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,rwork,info,[n,ncv,ldv,lworkl])\n\nWrapper for ``cnaupd``.\n\nParameters\n----------\nido : input int\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('F') with bounds (n)\nv : input rank-2 array('F') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : in/output rank-1 array('F') with bounds (3 * n)\nworkl : in/output rank-1 array('F') with bounds (lworkl)\nrwork : in/output rank-1 array('f') with bounds (ncv)\ninfo : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: shape(v,1)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nido : int\ntol : float\nresid : rank-1 array('F') with bounds (n)\nv : rank-2 array('F') with bounds (ldv,ncv)\niparam : rank-1 array('i') with bounds (11)\nipntr : rank-1 array('i') with bounds (14)\ninfo : int\n"
    ...

def cneupd(rvec, howmny, select, sigma, workev, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, rwork, info, ldz=..., n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "d,z,info = cneupd(rvec,howmny,select,sigma,workev,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,rwork,info,[ldz,n,ncv,ldv,lworkl])\n\nWrapper for ``cneupd``.\n\nParameters\n----------\nrvec : input int\nhowmny : input string(len=1)\nselect : input rank-1 array('i') with bounds (ncv)\nsigma : input complex\nworkev : input rank-1 array('F') with bounds (3 * ncv)\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('F') with bounds (n)\nv : input rank-2 array('F') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : input rank-1 array('F') with bounds (3 * n)\nworkl : input rank-1 array('F') with bounds (lworkl)\nrwork : input rank-1 array('f') with bounds (ncv)\ninfo : input int\n\nOther Parameters\n----------------\nldz : input int, optional\n    Default: shape(z,0)\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: len(select)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nd : rank-1 array('F') with bounds (nev)\nz : rank-2 array('F') with bounds (n,nev)\ninfo : int\n"
    ...

def debug() -> typing.Any:
    "'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n"
    ...

def dnaupd(ido, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "ido,tol,resid,v,iparam,ipntr,info = dnaupd(ido,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[n,ncv,ldv,lworkl])\n\nWrapper for ``dnaupd``.\n\nParameters\n----------\nido : input int\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('d') with bounds (n)\nv : input rank-2 array('d') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : in/output rank-1 array('d') with bounds (3 * n)\nworkl : in/output rank-1 array('d') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: shape(v,1)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nido : int\ntol : float\nresid : rank-1 array('d') with bounds (n)\nv : rank-2 array('d') with bounds (ldv,ncv)\niparam : rank-1 array('i') with bounds (11)\nipntr : rank-1 array('i') with bounds (14)\ninfo : int\n"
    ...

def dneupd(rvec, howmny, select, sigmar, sigmai, workev, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, ldz=..., n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "dr,di,z,info = dneupd(rvec,howmny,select,sigmar,sigmai,workev,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[ldz,n,ncv,ldv,lworkl])\n\nWrapper for ``dneupd``.\n\nParameters\n----------\nrvec : input int\nhowmny : input string(len=1)\nselect : input rank-1 array('i') with bounds (ncv)\nsigmar : input float\nsigmai : input float\nworkev : input rank-1 array('d') with bounds (3 * ncv)\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('d') with bounds (n)\nv : input rank-2 array('d') with bounds (n,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : input rank-1 array('d') with bounds (3 * n)\nworkl : input rank-1 array('d') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nldz : input int, optional\n    Default: shape(z,0)\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: len(select)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\ndr : rank-1 array('d') with bounds (nev + 1)\ndi : rank-1 array('d') with bounds (nev + 1)\nz : rank-2 array('d') with bounds (n,nev + 1)\ninfo : int\n"
    ...

def dsaupd(ido, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "ido,tol,resid,v,iparam,ipntr,info = dsaupd(ido,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[n,ncv,ldv,lworkl])\n\nWrapper for ``dsaupd``.\n\nParameters\n----------\nido : input int\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('d') with bounds (n)\nv : input rank-2 array('d') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (11)\nworkd : in/output rank-1 array('d') with bounds (3 * n)\nworkl : in/output rank-1 array('d') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: shape(v,1)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nido : int\ntol : float\nresid : rank-1 array('d') with bounds (n)\nv : rank-2 array('d') with bounds (ldv,ncv)\niparam : rank-1 array('i') with bounds (11)\nipntr : rank-1 array('i') with bounds (11)\ninfo : int\n"
    ...

def dseupd(rvec, howmny, select, sigma, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, ldz=..., n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "d,z,info = dseupd(rvec,howmny,select,sigma,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[ldz,n,ncv,ldv,lworkl])\n\nWrapper for ``dseupd``.\n\nParameters\n----------\nrvec : input int\nhowmny : input string(len=1)\nselect : input rank-1 array('i') with bounds (ncv)\nsigma : input float\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('d') with bounds (n)\nv : input rank-2 array('d') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (7)\nipntr : input rank-1 array('i') with bounds (11)\nworkd : input rank-1 array('d') with bounds (2 * n)\nworkl : input rank-1 array('d') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nldz : input int, optional\n    Default: shape(z,0)\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: len(select)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nd : rank-1 array('d') with bounds (nev)\nz : rank-2 array('d') with bounds (n,nev)\ninfo : int\n"
    ...

def snaupd(ido, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "ido,tol,resid,v,iparam,ipntr,info = snaupd(ido,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[n,ncv,ldv,lworkl])\n\nWrapper for ``snaupd``.\n\nParameters\n----------\nido : input int\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('f') with bounds (n)\nv : input rank-2 array('f') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : in/output rank-1 array('f') with bounds (3 * n)\nworkl : in/output rank-1 array('f') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: shape(v,1)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nido : int\ntol : float\nresid : rank-1 array('f') with bounds (n)\nv : rank-2 array('f') with bounds (ldv,ncv)\niparam : rank-1 array('i') with bounds (11)\nipntr : rank-1 array('i') with bounds (14)\ninfo : int\n"
    ...

def sneupd(rvec, howmny, select, sigmar, sigmai, workev, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, ldz=..., n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "dr,di,z,info = sneupd(rvec,howmny,select,sigmar,sigmai,workev,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[ldz,n,ncv,ldv,lworkl])\n\nWrapper for ``sneupd``.\n\nParameters\n----------\nrvec : input int\nhowmny : input string(len=1)\nselect : input rank-1 array('i') with bounds (ncv)\nsigmar : input float\nsigmai : input float\nworkev : input rank-1 array('f') with bounds (3 * ncv)\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('f') with bounds (n)\nv : input rank-2 array('f') with bounds (n,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : input rank-1 array('f') with bounds (3 * n)\nworkl : input rank-1 array('f') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nldz : input int, optional\n    Default: shape(z,0)\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: len(select)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\ndr : rank-1 array('f') with bounds (nev + 1)\ndi : rank-1 array('f') with bounds (nev + 1)\nz : rank-2 array('f') with bounds (n,nev + 1)\ninfo : int\n"
    ...

def ssaupd(ido, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "ido,tol,resid,v,iparam,ipntr,info = ssaupd(ido,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[n,ncv,ldv,lworkl])\n\nWrapper for ``ssaupd``.\n\nParameters\n----------\nido : input int\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('f') with bounds (n)\nv : input rank-2 array('f') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (11)\nworkd : in/output rank-1 array('f') with bounds (3 * n)\nworkl : in/output rank-1 array('f') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: shape(v,1)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nido : int\ntol : float\nresid : rank-1 array('f') with bounds (n)\nv : rank-2 array('f') with bounds (ldv,ncv)\niparam : rank-1 array('i') with bounds (11)\nipntr : rank-1 array('i') with bounds (11)\ninfo : int\n"
    ...

def sseupd(rvec, howmny, select, sigma, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, info, ldz=..., n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "d,z,info = sseupd(rvec,howmny,select,sigma,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,info,[ldz,n,ncv,ldv,lworkl])\n\nWrapper for ``sseupd``.\n\nParameters\n----------\nrvec : input int\nhowmny : input string(len=1)\nselect : input rank-1 array('i') with bounds (ncv)\nsigma : input float\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('f') with bounds (n)\nv : input rank-2 array('f') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (7)\nipntr : input rank-1 array('i') with bounds (11)\nworkd : input rank-1 array('f') with bounds (2 * n)\nworkl : input rank-1 array('f') with bounds (lworkl)\ninfo : input int\n\nOther Parameters\n----------------\nldz : input int, optional\n    Default: shape(z,0)\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: len(select)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nd : rank-1 array('f') with bounds (nev)\nz : rank-2 array('f') with bounds (n,nev)\ninfo : int\n"
    ...

def timing() -> typing.Any:
    "'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'i'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n'f'-scalar\n"
    ...

def znaupd(ido, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, rwork, info, n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "ido,tol,resid,v,iparam,ipntr,info = znaupd(ido,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,rwork,info,[n,ncv,ldv,lworkl])\n\nWrapper for ``znaupd``.\n\nParameters\n----------\nido : input int\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('D') with bounds (n)\nv : input rank-2 array('D') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : in/output rank-1 array('D') with bounds (3 * n)\nworkl : in/output rank-1 array('D') with bounds (lworkl)\nrwork : in/output rank-1 array('d') with bounds (ncv)\ninfo : input int\n\nOther Parameters\n----------------\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: shape(v,1)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nido : int\ntol : float\nresid : rank-1 array('D') with bounds (n)\nv : rank-2 array('D') with bounds (ldv,ncv)\niparam : rank-1 array('i') with bounds (11)\nipntr : rank-1 array('i') with bounds (14)\ninfo : int\n"
    ...

def zneupd(rvec, howmny, select, sigma, workev, bmat, which, nev, tol, resid, v, iparam, ipntr, workd, workl, rwork, info, ldz=..., n=..., ncv=..., ldv=..., lworkl=...) -> typing.Any:
    "d,z,info = zneupd(rvec,howmny,select,sigma,workev,bmat,which,nev,tol,resid,v,iparam,ipntr,workd,workl,rwork,info,[ldz,n,ncv,ldv,lworkl])\n\nWrapper for ``zneupd``.\n\nParameters\n----------\nrvec : input int\nhowmny : input string(len=1)\nselect : input rank-1 array('i') with bounds (ncv)\nsigma : input complex\nworkev : input rank-1 array('D') with bounds (3 * ncv)\nbmat : input string(len=1)\nwhich : input string(len=2)\nnev : input int\ntol : input float\nresid : input rank-1 array('D') with bounds (n)\nv : input rank-2 array('D') with bounds (ldv,ncv)\niparam : input rank-1 array('i') with bounds (11)\nipntr : input rank-1 array('i') with bounds (14)\nworkd : input rank-1 array('D') with bounds (3 * n)\nworkl : input rank-1 array('D') with bounds (lworkl)\nrwork : input rank-1 array('d') with bounds (ncv)\ninfo : input int\n\nOther Parameters\n----------------\nldz : input int, optional\n    Default: shape(z,0)\nn : input int, optional\n    Default: len(resid)\nncv : input int, optional\n    Default: len(select)\nldv : input int, optional\n    Default: shape(v,0)\nlworkl : input int, optional\n    Default: len(workl)\n\nReturns\n-------\nd : rank-1 array('D') with bounds (nev)\nz : rank-2 array('D') with bounds (n,nev)\ninfo : int\n"
    ...

def __getattr__(name) -> typing.Any:
    ...




from typing import Any, Callable, ClassVar, Sequence
from numpy.random import RandomState
from .fixes import threadpool_info as threadpool_info
from ..metrics import accuracy_score as accuracy_score, r2_score as r2_score
from .validation import (
    check_array as check_array,
    check_is_fitted as check_is_fitted,
    check_X_y as check_X_y,
)
from unittest import TestCase as TestCase
from numpy import ndarray
from numpydoc import docscrape as docscrape
from subprocess import (
    check_output as check_output,
    STDOUT as STDOUT,
    CalledProcessError as CalledProcessError,
    TimeoutExpired as TimeoutExpired,
)
from functools import wraps as wraps
from numpy.testing import (
    assert_allclose as np_assert_allclose,
    assert_almost_equal,
    assert_approx_equal,
    assert_array_equal,
    assert_array_almost_equal,
    assert_array_less,
)
from . import IS_PYPY as IS_PYPY
from .multiclass import check_classification_targets as check_classification_targets
from inspect import signature as signature
from .._typing import ArrayLike, MatrixLike, Float, Int
from collections.abc import Iterable as Iterable, Sequence

# Copyright (c) 2011, 2012
# Authors: Pietro Berkes,
#          Andreas Muller
#          Mathieu Blondel
#          Olivier Grisel
#          Arnaud Joly
#          Denis Engemann
#          Giorgio Patrini
#          Thierry Guillemot
# License: BSD 3 clause
import os
import os.path as op
import inspect
import warnings
import sys
import functools
import tempfile
import re
import contextlib

import scipy as sp

import shutil
import atexit
import unittest
import numpy as np
import joblib

import sklearn


__all__ = [
    "assert_raises",
    "assert_raises_regexp",
    "assert_array_equal",
    "assert_almost_equal",
    "assert_array_almost_equal",
    "assert_array_less",
    "assert_approx_equal",
    "assert_allclose",
    "assert_run_python_script",
    "SkipTest",
]

_dummy = ...
assert_raises = ...
SkipTest = ...
assert_dict_equal = ...

assert_raises_regex = ...
# assert_raises_regexp is deprecated in Python 3.4 in favor of
# assert_raises_regex but lets keep the backward compat in scikit-learn with
# the old name for now
assert_raises_regexp = ...


# To remove when we support numpy 1.7
def assert_no_warnings(func, *args, **kw):
    ...


def ignore_warnings(
    obj: None | Callable = None, category: Warning = ...
) -> _IgnoreWarnings | Callable:
    ...


class _IgnoreWarnings:
    def __init__(self, category: tuple[Warning]) -> None:
        ...

    def __call__(self, fn: Callable) -> Callable:
        ...

    def __repr__(self) -> str:
        ...

    def __enter__(self) -> None:
        ...

    def __exit__(self, *exc_info) -> None:
        ...


def assert_raise_message(
    exceptions: tuple[Exception, ...] | Exception,
    message: str,
    function: Callable,
    *args,
    **kwargs
):
    ...


def assert_allclose(
    actual: ArrayLike,
    desired: ArrayLike,
    rtol: float | None = None,
    atol: float | None = 0.0,
    equal_nan: None | bool = True,
    err_msg: None | str = "",
    verbose: None | bool = True,
):
    ...


def assert_allclose_dense_sparse(
    x: MatrixLike | ArrayLike,
    y: MatrixLike | ArrayLike,
    rtol: Float = 1e-07,
    atol: Float = 1e-9,
    err_msg: str = "",
):
    ...


def set_random_state(estimator: Any, random_state: RandomState | None | Int = 0):
    ...


def check_skip_network():
    ...


class TempMemmap:
    def __init__(self, data, mmap_mode: str = "r") -> None:
        ...

    def __enter__(self):
        ...

    def __exit__(self, exc_type, exc_val, exc_tb):
        ...


def create_memmap_backed_data(
    data, mmap_mode: str = "r", return_folder: bool = False, aligned: bool = False
):
    ...


def check_docstring_parameters(
    func: Callable, doc: None | str = None, ignore: Sequence | None = None
) -> ndarray:
    ...


def assert_run_python_script(source_code: str, timeout: Int = 60):
    ...


def raises(
    expected_exc_type,
    match: None | str | Sequence[str] = None,
    may_pass: bool = False,
    err_msg: None | str = None,
):
    ...


class _Raises(contextlib.AbstractContextManager):
    # see raises() for parameters
    def __init__(self, expected_exc_type, match, may_pass, err_msg) -> None:
        ...

    def __exit__(self, exc_type, exc_value, _):
        ...


class MinimalClassifier:

    _estimator_type: ClassVar[str] = ...

    def __init__(self, param=None) -> None:
        ...

    def get_params(self, deep: bool = True):
        ...

    def set_params(self, **params):
        ...

    def fit(self, X, y):
        ...

    def predict_proba(self, X):
        ...

    def predict(self, X):
        ...

    def score(self, X, y):
        ...


class MinimalRegressor:

    _estimator_type: ClassVar[str] = ...

    def __init__(self, param=None) -> None:
        ...

    def get_params(self, deep: bool = True):
        ...

    def set_params(self, **params):
        ...

    def fit(self, X, y):
        ...

    def predict(self, X):
        ...

    def score(self, X, y):
        ...


class MinimalTransformer:
    def __init__(self, param=None) -> None:
        ...

    def get_params(self, deep: bool = True):
        ...

    def set_params(self, **params):
        ...

    def fit(self, X, y=None):
        ...

    def transform(self, X, y=None):
        ...

    def fit_transform(self, X, y=None):
        ...



from typing import Callable, ClassVar, Iterable, Literal, Mapping, TypeVar
from numpy.random import RandomState
from scipy import linalg, sparse, optimize as optimize
from ..exceptions import ConvergenceWarning as ConvergenceWarning
from scipy.sparse._coo import coo_matrix
from ..utils.extmath import safe_sparse_dot as safe_sparse_dot, row_norms as row_norms
from ..model_selection._split import BaseShuffleSplit
from pandas.core.frame import DataFrame
from scipy.sparse._csr import csr_matrix
from ..metrics import (
    check_scoring as check_scoring,
    get_scorer_names as get_scorer_names,
)
from ..preprocessing import LabelBinarizer as LabelBinarizer
from ..utils.sparsefuncs import mean_variance_axis as mean_variance_axis
from scipy.sparse import linalg as sp_linalg
from ..utils.validation import check_is_fitted as check_is_fitted
from scipy.sparse.linalg import LinearOperator
from abc import ABCMeta, abstractmethod
from numpy import ndarray
from ..utils._param_validation import Interval as Interval, StrOptions as StrOptions
from numbers import Integral as Integral, Real as Real
from ._sag import sag_solver as sag_solver
from functools import partial as partial
from ..base import MultiOutputMixin, RegressorMixin, is_classifier as is_classifier
from ..model_selection import GridSearchCV as GridSearchCV
from ..utils import (
    check_array as check_array,
    check_consistent_length as check_consistent_length,
    check_scalar as check_scalar,
    compute_sample_weight as compute_sample_weight,
    column_or_1d as column_or_1d,
)
from ._base import LinearClassifierMixin, LinearModel
from pandas.core.series import Series
from .._typing import MatrixLike, ArrayLike, Int, Float
from ..model_selection import BaseCrossValidator

_BaseRidgeCV_Self = TypeVar("_BaseRidgeCV_Self", bound="_BaseRidgeCV")
_RidgeGCV_Self = TypeVar("_RidgeGCV_Self", bound="_RidgeGCV")
RidgeClassifier_Self = TypeVar("RidgeClassifier_Self", bound="RidgeClassifier")
RidgeClassifierCV_Self = TypeVar("RidgeClassifierCV_Self", bound="RidgeClassifierCV")
RidgeCV_Self = TypeVar("RidgeCV_Self", bound="RidgeCV")
Ridge_Self = TypeVar("Ridge_Self", bound="Ridge")

import warnings

import numpy as np
import numbers


def ridge_regression(
    X: MatrixLike | LinearOperator,
    y: MatrixLike | ArrayLike,
    alpha: float | ArrayLike,
    *,
    sample_weight: None | ArrayLike = None,
    solver: Literal[
        "auto", "svd", "cholesky", "lsqr", "sparse_cg", "sag", "saga", "lbfgs", "auto"
    ] = "auto",
    max_iter: None | Int = None,
    tol: Float = 1e-4,
    verbose: Int = 0,
    positive: bool = False,
    random_state: RandomState | None | Int = None,
    return_n_iter: bool = False,
    return_intercept: bool = False,
    check_input: bool = True,
) -> tuple[ndarray, int, float | ndarray]:
    ...


class _BaseRidge(LinearModel, metaclass=ABCMeta):

    _parameter_constraints: ClassVar[dict] = ...

    @abstractmethod
    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        fit_intercept: bool = True,
        copy_X: bool = True,
        max_iter=None,
        tol: float = 1e-4,
        solver: str = "auto",
        positive: bool = False,
        random_state=None,
    ) -> None:
        ...

    def fit(
        self,
        X: csr_matrix | ndarray,
        y: ndarray,
        sample_weight: Series | None | ndarray = None,
    ):
        ...


class Ridge(MultiOutputMixin, RegressorMixin, _BaseRidge):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    n_iter_: None | ndarray = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...

    def __init__(
        self,
        alpha: ArrayLike | Float = 1.0,
        *,
        fit_intercept: bool = True,
        copy_X: bool = True,
        max_iter: None | Int = None,
        tol: Float = 1e-4,
        solver: Literal[
            "auto",
            "svd",
            "cholesky",
            "lsqr",
            "sparse_cg",
            "sag",
            "saga",
            "lbfgs",
            "auto",
        ] = "auto",
        positive: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: Ridge_Self,
        X: coo_matrix | MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: float | None | ArrayLike = None,
    ) -> Ridge_Self:
        ...


class _RidgeClassifierMixin(LinearClassifierMixin):
    def predict(self, X: MatrixLike) -> ndarray:
        ...

    def classes_(self) -> ndarray:
        ...


class RidgeClassifier(_RidgeClassifierMixin, _BaseRidge):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    n_iter_: None | ndarray = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        alpha: Float = 1.0,
        *,
        fit_intercept: bool = True,
        copy_X: bool = True,
        max_iter: None | Int = None,
        tol: Float = 1e-4,
        class_weight: None | Mapping | str = None,
        solver: Literal[
            "auto",
            "svd",
            "cholesky",
            "lsqr",
            "sparse_cg",
            "sag",
            "saga",
            "lbfgs",
            "auto",
        ] = "auto",
        positive: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def fit(
        self: RidgeClassifier_Self,
        X: MatrixLike,
        y: ArrayLike,
        sample_weight: float | None | ArrayLike = None,
    ) -> RidgeClassifier_Self:
        ...


class _X_CenterStackOp(LinearOperator):
    def __init__(self, X, X_mean, sqrt_sw) -> None:
        ...


class _XT_CenterStackOp(LinearOperator):
    def __init__(self, X, X_mean, sqrt_sw) -> None:
        ...


class _IdentityRegressor:
    def decision_function(self, y_predict):
        ...

    def predict(self, y_predict):
        ...


class _IdentityClassifier(LinearClassifierMixin):
    def __init__(self, classes) -> None:
        ...

    def decision_function(self, y_predict) -> ndarray:
        ...


class _RidgeGCV(LinearModel):
    def __init__(
        self,
        alphas: ndarray = ...,
        *,
        fit_intercept: bool = True,
        scoring=None,
        copy_X: bool = True,
        gcv_mode=None,
        store_cv_values: bool = False,
        is_clf: bool = False,
        alpha_per_target: bool = False,
    ) -> None:
        ...

    def fit(
        self: _RidgeGCV_Self,
        X: MatrixLike,
        y: MatrixLike | ArrayLike,
        sample_weight: float | None | ArrayLike = None,
    ) -> _RidgeGCV_Self:
        ...


class _BaseRidgeCV(LinearModel):

    _parameter_constraints: ClassVar[dict] = ...

    def __init__(
        self,
        alphas: tuple[float, float, float] | ndarray = ...,
        *,
        fit_intercept: bool = True,
        scoring=None,
        cv=None,
        gcv_mode=None,
        store_cv_values: bool = False,
        alpha_per_target: bool = False,
    ) -> None:
        ...

    def fit(
        self: _BaseRidgeCV_Self,
        X: ArrayLike | DataFrame,
        y: MatrixLike | ArrayLike,
        sample_weight: float | None | ArrayLike = None,
    ) -> _BaseRidgeCV_Self | RidgeCV:
        ...


class RidgeCV(MultiOutputMixin, RegressorMixin, _BaseRidgeCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    best_score_: float | ndarray = ...
    alpha_: float | ndarray = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...
    cv_values_: ndarray = ...

    def fit(
        self: RidgeCV_Self,
        X: ArrayLike | DataFrame,
        y: MatrixLike | ArrayLike,
        sample_weight: float | None | ArrayLike = None,
    ) -> RidgeCV_Self:
        ...


class RidgeClassifierCV(_RidgeClassifierMixin, _BaseRidgeCV):
    feature_names_in_: ndarray = ...
    n_features_in_: int = ...
    classes_: ndarray = ...
    best_score_: float = ...
    alpha_: float = ...
    intercept_: float | ndarray = ...
    coef_: ndarray = ...
    cv_values_: ndarray = ...

    _parameter_constraints: ClassVar[dict] = ...
    for param in ("gcv_mode", "alpha_per_target"):
        pass

    def __init__(
        self,
        alphas: ArrayLike = ...,
        *,
        fit_intercept: bool = True,
        scoring: None | str | Callable = None,
        cv: int | BaseCrossValidator | Iterable | None | BaseShuffleSplit = None,
        class_weight: None | Mapping | str = None,
        store_cv_values: bool = False,
    ) -> None:
        ...

    def fit(
        self: RidgeClassifierCV_Self,
        X: ArrayLike,
        y: ArrayLike,
        sample_weight: float | None | ArrayLike = None,
    ) -> RidgeClassifierCV_Self:
        ...



"""
This type stub file was generated by pyright.
"""

"""Find files and directories which IPython uses.
"""
def get_ipython_dir() -> str:
    """Get the IPython directory for this platform and user.

    This uses the logic in `get_home_dir` to find the home directory
    and then adds .ipython to the end of the path.
    """
    ...

def get_ipython_cache_dir() -> str:
    """Get the cache directory it is created if it does not exist."""
    ...

def get_ipython_package_dir() -> str:
    """Get the base directory where IPython itself is installed."""
    ...

def get_ipython_module_path(module_str):
    """Find the path to an IPython module in this version of IPython.

    This will always find the version of the module that is in this importable
    IPython package. This will always return the path to the ``.py``
    version of the module.
    """
    ...

def locate_profile(profile=...):
    """Find the path to the folder associated with a given profile.

    I.e. find $IPYTHONDIR/profile_whatever.
    """
    ...




from .common import Y_DTYPE as Y_DTYPE
from ..._typing import ArrayLike, MatrixLike, Int
from numpy import ndarray

# Author: Nicolas Hug

import numpy as np


class TreePredictor:
    def __init__(
        self,
        nodes: ArrayLike,
        binned_left_cat_bitsets: MatrixLike,
        raw_left_cat_bitsets: MatrixLike,
    ) -> None:
        ...

    def get_n_leaf_nodes(self):
        ...

    def get_max_depth(self):
        ...

    def predict(
        self,
        X: MatrixLike,
        known_cat_bitsets: MatrixLike,
        f_idx_map: ArrayLike,
        n_threads: Int,
    ) -> ndarray:
        ...

    def predict_binned(
        self, X: MatrixLike, missing_values_bin_idx: int, n_threads: Int
    ) -> ndarray:
        ...

    def compute_partial_dependence(
        self, grid: MatrixLike, target_features: ArrayLike, out: ArrayLike
    ) -> None:
        ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

from ._testing import (
    SkipTest as SkipTest,
    requires_application as requires_application,
    requires_ipython as requires_ipython,  # noqa
    requires_img_lib as requires_img_lib,  # noqa
    requires_pyopengl as requires_pyopengl,  # noqa
    requires_scipy as requires_scipy,  # noqa
    save_testing_image as save_testing_image,
    TestingCanvas as TestingCanvas,
    has_pyopengl as has_pyopengl,  # noqa
    run_tests_if_main as run_tests_if_main,
    requires_ssl as requires_ssl,  # noqa
    assert_is as assert_is,
    assert_in as assert_in,
    assert_not_in as assert_not_in,
    assert_equal as assert_equal,
    assert_not_equal as assert_not_equal,
    assert_raises as assert_raises,
    assert_true as assert_true,  # noqa
    raises as raises,
    requires_numpydoc as requires_numpydoc,
    IS_TRAVIS_CI as IS_TRAVIS_CI,
    IS_CI as IS_CI,
)  # noqa
from ._runners import test as test  # noqa



from .misc import (
    is_string_like as is_string_like,
    iterable as iterable,
    empty_generator as empty_generator,
    flatten as flatten,
    make_list_of_ints as make_list_of_ints,
    is_list_of_ints as is_list_of_ints,
    make_str as make_str,
    generate_unique_node as generate_unique_node,
    default_opener as default_opener,
    dict_to_numpy_array as dict_to_numpy_array,
    dict_to_numpy_array1 as dict_to_numpy_array1,
    dict_to_numpy_array2 as dict_to_numpy_array2,
    is_iterator as is_iterator,
    arbitrary_element as arbitrary_element,
    consume as consume,
    pairwise as pairwise,
    groups as groups,
    to_tuple as to_tuple,
    create_random_state as create_random_state,
    create_py_random_state as create_py_random_state,
    PythonRandomInterface as PythonRandomInterface,
    nodes_equal as nodes_equal,
    edges_equal as edges_equal,
    graphs_equal as graphs_equal,
)
from .decorators import (
    not_implemented_for as not_implemented_for,
    open_file as open_file,
    nodes_or_number as nodes_or_number,
    preserve_random_state as preserve_random_state,
    random_state as random_state,
    np_random_state as np_random_state,
    py_random_state as py_random_state,
    argmap as argmap,
)
from .random_sequence import (
    powerlaw_sequence as powerlaw_sequence,
    zipf_rv as zipf_rv,
    cumulative_distribution as cumulative_distribution,
    discrete_sequence as discrete_sequence,
    random_weighted_sample as random_weighted_sample,
    weighted_choice as weighted_choice,
)
from .union_find import *
from .rcm import (
    cuthill_mckee_ordering as cuthill_mckee_ordering,
    reverse_cuthill_mckee_ordering as reverse_cuthill_mckee_ordering,
)
from .heaps import MinHeap as MinHeap, PairingHeap as PairingHeap, BinaryHeap as BinaryHeap
from .contextmanagers import reversed as reversed



import numpy as np

get_window_bounds_doc: str

class BaseIndexer:
    index_array = ...
    window_size = ...
    def __init__(
        self, index_array: np.ndarray | None = ..., window_size: int = ..., **kwargs
    ) -> None: ...
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...

class FixedWindowIndexer(BaseIndexer):
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...

class VariableWindowIndexer(BaseIndexer):
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...

class VariableOffsetWindowIndexer(BaseIndexer):
    def __init__(
        self,
        index_array: np.ndarray | None = ...,
        window_size: int = ...,
        index=...,
        offset=...,
        **kwargs,
    ) -> None: ...
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...

class ExpandingIndexer(BaseIndexer):
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...

class FixedForwardWindowIndexer(BaseIndexer):
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...

class GroupbyRollingIndexer(BaseIndexer):
    def __init__(
        self,
        index_array: np.ndarray | None,
        window_size: int,
        groupby_indicies: dict,
        rolling_indexer: type[FixedWindowIndexer] | type[VariableWindowIndexer],
        **kwargs,
    ) -> None: ...
    def get_window_bounds(
        self,
        num_values: int = ...,
        min_periods: int | None = ...,
        center: bool | None = ...,
        closed: str | None = ...,
    ) -> tuple[np.ndarray, np.ndarray]: ...



"""
This type stub file was generated by pyright.
"""

import os

"""
Paging capabilities for IPython.core

Notes
-----

For now this uses IPython hooks, so it can't be in IPython.utils.  If we can get
rid of that dependency, we could move it there.
-----
"""
def display_page(strng, start=..., screen_lines=...): # -> None:
    """Just display, no paging. screen_lines is ignored."""
    ...

def as_hook(page_func): # -> (self: Unknown, *args: Unknown, **kwargs: Unknown) -> Unknown:
    """Wrap a pager func to strip the `self` arg

    so it can be called as a hook.
    """
    ...

esc_re = ...
def page_dumb(strng, start=..., screen_lines=...): # -> None:
    """Very dumb 'pager' in Python, for when nothing else works.

    Only moves forward, same interface as page(), except for pager_cmd and
    mode.
    """
    ...

def pager_page(strng, start=..., screen_lines=..., pager_cmd=...): # -> None:
    """Display a string, piping through a pager after a certain length.

    strng can be a mime-bundle dict, supplying multiple representations,
    keyed by mime-type.

    The screen_lines parameter specifies the number of *usable* lines of your
    terminal screen (total lines minus lines you need to reserve to show other
    information).

    If you set screen_lines to a number <=0, page() will try to auto-determine
    your screen size and will only use up to (screen_size+screen_lines) for
    printing, paging after that. That is, if you want auto-detection but need
    to reserve the bottom 3 lines of the screen, use screen_lines = -3, and for
    auto-detection without any lines reserved simply use screen_lines = 0.

    If a string won't fit in the allowed lines, it is sent through the
    specified pager command. If none given, look for PAGER in the environment,
    and ultimately default to less.

    If no system pager works, the string is sent through a 'dumb pager'
    written in python, very simplistic.
    """
    ...

def page(data, start=..., screen_lines=..., pager_cmd=...): # -> None:
    """Display content in a pager, piping through a pager after a certain length.

    data can be a mime-bundle dict, supplying multiple representations,
    keyed by mime-type, or text.

    Pager is dispatched via the `show_in_pager` IPython hook.
    If no hook is registered, `pager_page` will be used.
    """
    ...

def page_file(fname, start=..., pager_cmd=...): # -> None:
    """Page a file, using an optional pager command and starting line.
    """
    ...

def get_pager_cmd(pager_cmd=...):
    """Return a pager command.

    Makes some attempts at finding an OS-correct one.
    """
    ...

def get_pager_start(pager, start):
    """Return the string for paging files with an offset.

    This is the '+N' argument which less and more (under Unix) accept.
    """
    ...

if os.name == 'nt' and os.environ.get('TERM', 'dumb') != 'emacs':
    def page_more():
        """ Smart pausing between pages

        @return:    True if need print more lines, False if quit
        """
        ...
    
else:
    def page_more():
        ...
    



from ..classes.multigraph import MultiGraph
from ..classes.graph import Graph
from ..utils import py_random_state

__all__ = ["random_clustered_graph"]

@py_random_state(2)
def random_clustered_graph(
    joint_degree_sequence, create_using=None, seed=None
) -> MultiGraph: ...



from datetime import datetime, timedelta
from decimal import Decimal
from typing import (
    Any,
    Callable,
    Dict,
    Iterable,
    Iterator,
    List,
    Optional,
    Sequence,
    Set,
    Tuple,
    Type,
    TypeVar,
    Union,
)

from django.db.models import Q, QuerySet
from django.db.models.fields import Field
from django.db.models.lookups import Lookup
from django.db.models.query import _BaseQuerySet
from django.db.models.sql.compiler import SQLCompiler

_OutputField = Union[Field[Any, Any], str]

class SQLiteNumericMixin:
    def as_sqlite(
        self, compiler: SQLCompiler, connection: Any, **extra_context: Any
    ) -> Tuple[str, List[float]]: ...

_Self = TypeVar("_Self")

_Numeric = Union[float, Decimal]

class Combinable:
    ADD: str = ...
    SUB: str = ...
    MUL: str = ...
    DIV: str = ...
    POW: str = ...
    MOD: str = ...
    BITAND: str = ...
    BITOR: str = ...
    BITLEFTSHIFT: str = ...
    BITRIGHTSHIFT: str = ...
    def __neg__(self: _Self) -> _Self: ...
    def __add__(
        self: _Self, other: Optional[Union[timedelta, Combinable, _Numeric, str]]
    ) -> _Self: ...
    def __sub__(
        self: _Self, other: Union[timedelta, Combinable, _Numeric]
    ) -> _Self: ...
    def __mul__(
        self: _Self, other: Union[timedelta, Combinable, _Numeric]
    ) -> _Self: ...
    def __truediv__(self: _Self, other: Union[Combinable, _Numeric]) -> _Self: ...
    def __itruediv__(self: _Self, other: Union[Combinable, _Numeric]) -> _Self: ...
    def __mod__(self: _Self, other: Union[int, Combinable]) -> _Self: ...
    def __pow__(self: _Self, other: Union[_Numeric, Combinable]) -> _Self: ...
    def __and__(self: _Self, other: Combinable) -> _Self: ...
    def bitand(self: _Self, other: int) -> _Self: ...
    def bitleftshift(self: _Self, other: int) -> _Self: ...
    def bitrightshift(self: _Self, other: int) -> _Self: ...
    def __or__(self: _Self, other: Combinable) -> _Self: ...
    def bitor(self: _Self, other: int) -> _Self: ...
    def __radd__(
        self, other: Optional[Union[datetime, _Numeric, Combinable]]
    ) -> Combinable: ...
    def __rsub__(self, other: Union[_Numeric, Combinable]) -> Combinable: ...
    def __rmul__(self, other: Union[_Numeric, Combinable]) -> Combinable: ...
    def __rtruediv__(self, other: Union[_Numeric, Combinable]) -> Combinable: ...
    def __rmod__(self, other: Union[int, Combinable]) -> Combinable: ...
    def __rpow__(self, other: Union[_Numeric, Combinable]) -> Combinable: ...
    def __rand__(self, other: Any) -> Combinable: ...
    def __ror__(self, other: Any) -> Combinable: ...

class BaseExpression:
    is_summary: bool = ...
    filterable: bool = ...
    window_compatible: bool = ...
    def __init__(self, output_field: Optional[_OutputField] = ...) -> None: ...
    def get_db_converters(self, connection: Any) -> List[Callable[..., Any]]: ...
    def get_source_expressions(self) -> List[Any]: ...
    def set_source_expressions(self, exprs: Sequence[Combinable]) -> None: ...
    @property
    def contains_aggregate(self) -> bool: ...
    @property
    def contains_over_clause(self) -> bool: ...
    @property
    def contains_column_references(self) -> bool: ...
    def resolve_expression(
        self: _Self,
        query: Any = ...,
        allow_joins: bool = ...,
        reuse: Optional[Set[str]] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> _Self: ...
    @property
    def field(self) -> Field[Any, Any]: ...
    @property
    def output_field(self) -> Field[Any, Any]: ...
    @property
    def convert_value(self) -> Callable[..., Any]: ...
    def get_lookup(self, lookup: str) -> Optional[Type[Lookup[Any]]]: ...
    def get_transform(self, name: str) -> Optional[Type[Expression]]: ...
    def relabeled_clone(self, change_map: Dict[Optional[str], str]) -> Expression: ...
    def copy(self) -> BaseExpression: ...
    def get_group_by_cols(self: _Self) -> List[_Self]: ...
    def get_source_fields(self) -> List[Optional[Field[Any, Any]]]: ...
    def asc(self, **kwargs: Any) -> Expression: ...
    def desc(self, **kwargs: Any) -> Expression: ...
    def reverse_ordering(self) -> Any: ...
    def flatten(self) -> Iterator[Expression]: ...
    def deconstruct(self) -> Any: ...
    def as_sqlite(self, compiler: SQLCompiler, connection: Any) -> Any: ...
    def as_sql(
        self, compiler: SQLCompiler, connection: Any, **extra_context: Any
    ) -> Any: ...
    def as_mysql(self, compiler: Any, connection: Any) -> Any: ...
    def as_postgresql(self, compiler: Any, connection: Any) -> Any: ...
    def as_oracle(self, compiler: Any, connection: Any) -> Any: ...

class Expression(BaseExpression, Combinable): ...

class CombinedExpression(SQLiteNumericMixin, Expression):
    connector: Any = ...
    lhs: Any = ...
    rhs: Any = ...
    def __init__(
        self,
        lhs: Combinable,
        connector: str,
        rhs: Combinable,
        output_field: Optional[_OutputField] = ...,
    ) -> None: ...

class F(Combinable):
    name: str
    def __init__(self, name: str) -> None: ...
    def resolve_expression(
        self: _Self,
        query: Any = ...,
        allow_joins: bool = ...,
        reuse: Optional[Set[str]] = ...,
        summarize: bool = ...,
        for_save: bool = ...,
    ) -> _Self: ...
    def asc(self, **kwargs: Any) -> OrderBy: ...
    def desc(self, **kwargs: Any) -> OrderBy: ...
    def deconstruct(self) -> Any: ...

class OuterRef(F):
    def __init__(self, name: Union[str, OuterRef]) -> None: ...

class Subquery(Expression):
    template: str = ...
    queryset: QuerySet[Any] = ...
    extra: Dict[Any, Any] = ...
    def __init__(
        self,
        queryset: _BaseQuerySet[Any],
        output_field: Optional[_OutputField] = ...,
        **extra: Any
    ) -> None: ...

class Exists(Subquery):
    negated: bool = ...
    def __init__(self, *args: Any, negated: bool = ..., **kwargs: Any) -> None: ...
    def __invert__(self) -> Exists: ...

class OrderBy(BaseExpression):
    template: str = ...
    nulls_first: bool = ...
    nulls_last: bool = ...
    descending: bool = ...
    expression: Expression = ...
    def __init__(
        self,
        expression: Combinable,
        descending: bool = ...,
        nulls_first: bool = ...,
        nulls_last: bool = ...,
    ) -> None: ...

class Value(Expression):
    value: Any = ...
    def __init__(
        self, value: Any, output_field: Optional[_OutputField] = ...
    ) -> None: ...

class RawSQL(Expression):
    params: List[Any]
    sql: str
    def __init__(
        self,
        sql: str,
        params: Sequence[Any],
        output_field: Optional[_OutputField] = ...,
    ) -> None: ...

class Func(SQLiteNumericMixin, Expression):
    function: str = ...
    name: str = ...
    template: str = ...
    arg_joiner: str = ...
    arity: int = ...
    source_expressions: List[Combinable] = ...
    extra: Dict[Any, Any] = ...
    def __init__(
        self,
        *expressions: Any,
        output_field: Optional[_OutputField] = ...,
        **extra: Any
    ) -> None: ...

class When(Expression):
    template: str = ...
    condition: Any = ...
    result: Any = ...
    def __init__(
        self, condition: Any = ..., then: Any = ..., **lookups: Any
    ) -> None: ...

class Case(Expression):
    template: str = ...
    case_joiner: str = ...
    cases: Any = ...
    default: Any = ...
    extra: Any = ...
    def __init__(
        self,
        *cases: Any,
        default: Optional[Any] = ...,
        output_field: Optional[_OutputField] = ...,
        **extra: Any
    ) -> None: ...

class ExpressionWrapper(Expression):
    def __init__(
        self, expression: Union[Q, Combinable], output_field: _OutputField
    ) -> None: ...

class Col(Expression):
    def __init__(
        self, alias: str, target: str, output_field: Optional[_OutputField] = ...
    ) -> None: ...

class SimpleCol(Expression):
    contains_column_references: bool = ...
    def __init__(
        self, target: Field[Any, Any], output_field: Optional[_OutputField] = ...
    ) -> None: ...

class Ref(Expression):
    def __init__(self, refs: str, source: Expression) -> None: ...

class ExpressionList(Func):
    def __init__(
        self, *expressions: Union[BaseExpression, Combinable], **extra: Any
    ) -> None: ...

class Random(Expression): ...

class Window(Expression):
    template: str = ...
    contains_aggregate: bool = ...
    contains_over_clause: bool = ...
    def __init__(
        self,
        expression: BaseExpression,
        partition_by: Optional[
            Union[str, Iterable[Union[BaseExpression, F]], F, BaseExpression]
        ] = ...,
        order_by: Optional[
            Union[Sequence[Union[BaseExpression, F]], Union[BaseExpression, F]]
        ] = ...,
        frame: Optional[WindowFrame] = ...,
        output_field: Optional[_OutputField] = ...,
    ) -> None: ...

class WindowFrame(Expression):
    template: str = ...
    frame_type: str = ...
    def __init__(
        self, start: Optional[int] = ..., end: Optional[int] = ...
    ) -> None: ...
    def window_frame_start_end(
        self, connection: Any, start: Optional[int], end: Optional[int]
    ) -> Tuple[int, int]: ...

class RowRange(WindowFrame): ...
class ValueRange(WindowFrame): ...



from typing import Mapping
from collections import defaultdict
from itertools import combinations
from operator import itemgetter

from ...classes.graph import Graph

# Define the default maximum flow function.
from ...algorithms.flow import edmonds_karp
from ...utils import not_implemented_for

default_flow_func = ...

__all__ = ["k_components"]

def k_components(G: Graph, flow_func=None) -> Mapping: ...
def build_k_number_dict(kcomps): ...



from matplotlib.markers import MarkerStyle
import numpy as np
from typing import Any, Callable, Literal, Sequence
from ._typing import *
from .transforms import Bbox, Transform
from .path import Path
from .backend_bases import Event, MouseEvent, RendererBase
from ._enums import CapStyle, JoinStyle
from .artist import Artist, allow_rasterization
from .backends.backend_agg import RendererAgg

def segment_hits(cx, cy, x, y, radius): ...

class Line2D(Artist):

    lineStyles = ...

    drawStyles = ...
    drawStyleKeys = ...
    markers = ...
    filled_markers = ...
    fillStyles = ...
    zorder = ...
    def __str__(self) -> str: ...
    def __init__(
        self,
        xdata: Sequence[float],
        ydata: Sequence[float],
        linewidth: float = ...,
        linestyle=...,
        color: Color = ...,
        marker: str = ...,
        markersize: float = ...,
        markeredgewidth: float = ...,
        markeredgecolor: Color = ...,
        markerfacecolor: Color = ...,
        markerfacecoloralt: Color = ...,
        fillstyle: Literal["full", "left", "right", "bottom", "top", "none"] = ...,
        antialiased: bool = ...,
        dash_capstyle: CapStyle = ...,
        solid_capstyle: CapStyle = ...,
        dash_joinstyle: JoinStyle = ...,
        solid_joinstyle: JoinStyle = ...,
        pickradius: float = ...,
        drawstyle: Literal[
            "default", "steps", "steps-pre", "steps-mid", "steps-post"
        ] = "default",
        markevery=...,
        **kwargs
    ) -> None: ...
    def contains(self, mouseevent: MouseEvent): ...
    def get_pickradius(self) -> float: ...
    def set_pickradius(self, d: float) -> None: ...
    pickradius = ...
    def get_fillstyle(self) -> str: ...
    def set_fillstyle(
        self, fs: Literal["full", "left", "right", "bottom", "top", "none"]
    )-> None: ...
    def set_markevery(self, every) -> None: ...
    def get_markevery(self) -> Any: ...
    def set_picker(self, p: Callable | float) -> None: ...
    def get_bbox(self) -> Bbox: ...
    def get_window_extent(self, renderer: RendererBase = ...): ...
    def set_data(self, *args) -> None: ...
    def recache_always(self) -> bool: ...
    def recache(self, always: bool = ...) -> None: ...
    def set_transform(self, t: Transform): ...
    @allow_rasterization
    def draw(self, renderer: RendererBase): ...
    def get_antialiased(self) -> bool: ...
    def get_color(self) -> str: ...
    def get_drawstyle(self) -> str: ...
    def get_linestyle(self) -> str: ...
    def get_linewidth(self) -> float: ...
    def get_marker(self) -> int | str: ...
    def get_markeredgecolor(self) -> Color: ...
    def get_markeredgewidth(self) -> float: ...
    def get_markerfacecolor(self) -> Color: ...
    def get_markerfacecoloralt(self) -> Color: ...
    def get_markersize(self) -> float: ...
    def get_data(
        self, orig: bool = False
    ) -> tuple[Sequence[float], Sequence[float]]: ...
    def get_xdata(self, orig: bool = False) -> Sequence[float]: ...
    def get_ydata(self, orig: bool = False) -> Sequence[float]: ...
    def get_path(self) -> Path: ...
    def get_xydata(self) -> np.ndarray: ...
    def set_antialiased(self, b: bool) -> None: ...
    def set_color(self, color: Color) -> None: ...
    def set_drawstyle(
        self,
        drawstyle: Literal[
            "default", "steps", "steps-pre", "steps-mid", "steps-post"
        ] = "default",
    ) -> None: ...
    def set_linewidth(self, w: float) -> None: ...
    def set_linestyle(self, ls: Any) -> None: ...
    def set_marker(self, marker: str | Path | MarkerStyle) -> None: ...
    def set_markeredgecolor(self, ec: Color) -> None: ...
    def set_markerfacecolor(self, fc: Color) -> None: ...
    def set_markerfacecoloralt(self, fc: Color) -> None: ...
    def set_markeredgewidth(self, ew: float) -> None: ...
    def set_markersize(self, sz: float) -> None: ...
    def set_xdata(self, x: ArrayLike) -> None: ...
    def set_ydata(self, y: ArrayLike) -> None: ...
    def set_dashes(self, seq: Sequence[int]) -> None: ...
    def update_from(self, other: Line2D) -> None: ...
    def set_dash_joinstyle(self, s: JoinStyle) -> None: ...
    def set_solid_joinstyle(self, s: JoinStyle | str) -> None: ...
    def get_dash_joinstyle(self) -> JoinStyle | str: ...
    def get_solid_joinstyle(self) -> JoinStyle | str: ...
    def set_dash_capstyle(self, s: CapStyle | str) -> None: ...
    def set_solid_capstyle(self, s: CapStyle | str) -> None: ...
    def get_dash_capstyle(self) -> CapStyle | str: ...
    def get_solid_capstyle(self) -> CapStyle | str: ...
    def is_dashed(self) -> bool: ...

class _AxLine(Line2D):
    def __init__(
        self, xy1: Sequence[float], xy2: Sequence[float], slope: float, **kwargs
    ) -> None: ...
    def get_transform(self): ...
    def draw(self, renderer: RendererAgg) -> None: ...

class VertexSelector:
    def __init__(self, line: Line2D) -> None: ...
    def process_selected(self, ind: Sequence[int], xs: ArrayLike, ys: ArrayLike): ...
    def onpick(self, event: Event): ...

lineStyles = ...
lineMarkers = ...
drawStyles = ...
fillStyles = ...



from typing import Any, Callable, Iterable, Iterator
from numpy.random import RandomState
from ..utils import indexable as indexable, check_random_state as check_random_state
from scipy.special import comb as comb
from collections import defaultdict as defaultdict
from abc import ABCMeta, abstractmethod
from itertools import chain as chain, combinations as combinations
from numpy import ndarray
from ..utils.multiclass import type_of_target as type_of_target
from math import ceil as ceil, floor as floor
from inspect import signature as signature
from .._typing import MatrixLike, ArrayLike, Int, Float
from collections.abc import Iterable
from ..utils.validation import column_or_1d as column_or_1d, check_array as check_array
import warnings
import numbers

import numpy as np

__all__ = [
    "BaseCrossValidator",
    "KFold",
    "GroupKFold",
    "LeaveOneGroupOut",
    "LeaveOneOut",
    "LeavePGroupsOut",
    "LeavePOut",
    "RepeatedStratifiedKFold",
    "RepeatedKFold",
    "ShuffleSplit",
    "GroupShuffleSplit",
    "StratifiedKFold",
    "StratifiedGroupKFold",
    "StratifiedShuffleSplit",
    "PredefinedSplit",
    "train_test_split",
    "check_cv",
]


class BaseCrossValidator(metaclass=ABCMeta):
    def split(
        self,
        X: list[str] | MatrixLike,
        y: None | ArrayLike = None,
        groups: None | ArrayLike = None,
    ) -> Iterator[tuple[ndarray, ndarray]]:
        ...

    @abstractmethod
    def get_n_splits(self, X=None, y=None, groups=None):
        ...

    def __repr__(self) -> str:
        ...


class LeaveOneOut(BaseCrossValidator):
    def get_n_splits(self, X: MatrixLike, y: Any = None, groups: Any = None) -> int:
        ...


class LeavePOut(BaseCrossValidator):
    def __init__(self, p: Int) -> None:
        ...

    def get_n_splits(self, X: MatrixLike, y: Any = None, groups: Any = None):
        ...


class _BaseKFold(BaseCrossValidator, metaclass=ABCMeta):
    @abstractmethod
    def __init__(self, n_splits: int, *, shuffle, random_state) -> None:
        ...

    def split(
        self,
        X: list[str] | MatrixLike,
        y: None | ArrayLike = None,
        groups: None | ArrayLike = None,
    ) -> Iterator[tuple[ndarray, ndarray]]:
        ...

    def get_n_splits(self, X: Any = None, y: Any = None, groups: Any = None) -> int:
        ...


class KFold(_BaseKFold):
    def __init__(
        self,
        n_splits: Int = 5,
        *,
        shuffle: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class GroupKFold(_BaseKFold):
    def __init__(self, n_splits: Int = 5) -> None:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ) -> Iterator[Any]:
        ...


class StratifiedKFold(_BaseKFold):
    def __init__(
        self,
        n_splits: Int = 5,
        *,
        shuffle: bool = False,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def split(
        self, X: list[str] | MatrixLike, y: ArrayLike, groups: Any = None
    ) -> Iterator[Any]:
        ...


class StratifiedGroupKFold(_BaseKFold):
    def __init__(
        self,
        n_splits: Int = 5,
        shuffle: bool = False,
        random_state: None | RandomState | int = None,
    ) -> None:
        ...


class TimeSeriesSplit(_BaseKFold):
    def __init__(
        self,
        n_splits: Int = 5,
        *,
        max_train_size: None | Int = None,
        test_size: None | Int = None,
        gap: Int = 0,
    ) -> None:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ) -> Iterator[tuple[ndarray, ndarray]]:
        ...


class LeaveOneGroupOut(BaseCrossValidator):
    def get_n_splits(
        self, X: Any = None, y: Any = None, groups: None | ArrayLike = None
    ) -> int:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ):
        ...


class LeavePGroupsOut(BaseCrossValidator):
    def __init__(self, n_groups: Int) -> None:
        ...

    def get_n_splits(
        self, X: Any = None, y: Any = None, groups: None | ArrayLike = None
    ) -> int:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ):
        ...


class _RepeatedSplits(metaclass=ABCMeta):
    def __init__(
        self,
        cv: Callable,
        *,
        n_repeats: Int = 10,
        random_state: RandomState | None | Int = None,
        **cvargs,
    ) -> None:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ) -> Iterator[tuple[ndarray, ndarray]]:
        ...

    def get_n_splits(
        self, X: Any = None, y: Any = None, groups: None | ArrayLike = None
    ) -> int:
        ...

    def __repr__(self) -> str:
        ...


class RepeatedKFold(_RepeatedSplits):
    def __init__(
        self,
        *,
        n_splits: Int = 5,
        n_repeats: Int = 10,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class RepeatedStratifiedKFold(_RepeatedSplits):
    def __init__(
        self,
        *,
        n_splits: Int = 5,
        n_repeats: Int = 10,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class BaseShuffleSplit(metaclass=ABCMeta):
    def __init__(
        self, n_splits: int = 10, *, test_size=None, train_size=None, random_state=None
    ) -> None:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ) -> Iterator[tuple[ndarray, ndarray]]:
        ...

    def get_n_splits(self, X: Any = None, y: Any = None, groups: Any = None) -> int:
        ...

    def __repr__(self) -> str:
        ...


class ShuffleSplit(BaseShuffleSplit):
    def __init__(
        self,
        n_splits: Int = 10,
        *,
        test_size: None | Float = None,
        train_size: None | Float = None,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...


class GroupShuffleSplit(ShuffleSplit):
    def __init__(
        self,
        n_splits: Int = 5,
        *,
        test_size: float | None | int = None,
        train_size: None | Float = None,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def split(
        self, X: MatrixLike, y: None | ArrayLike = None, groups: None | ArrayLike = None
    ) -> Iterator[Any]:
        ...


class StratifiedShuffleSplit(BaseShuffleSplit):
    def __init__(
        self,
        n_splits: Int = 10,
        *,
        test_size: None | Float = None,
        train_size: None | Float = None,
        random_state: RandomState | None | Int = None,
    ) -> None:
        ...

    def split(self, X: MatrixLike, y: ArrayLike, groups: Any = None) -> Iterator[Any]:
        ...


class PredefinedSplit(BaseCrossValidator):
    def __init__(self, test_fold: ArrayLike) -> None:
        ...

    def split(self, X: Any = None, y: Any = None, groups: Any = None):
        ...

    def get_n_splits(self, X: Any = None, y: Any = None, groups: Any = None) -> int:
        ...


class _CVIterableWrapper(BaseCrossValidator):
    def __init__(self, cv) -> None:
        ...

    def get_n_splits(self, X: Any = None, y: Any = None, groups: Any = None) -> int:
        ...

    def split(self, X: Any = None, y: Any = None, groups: Any = None):
        ...


def check_cv(
    cv: Iterable | int | BaseShuffleSplit | BaseCrossValidator = 5,
    y: None | ArrayLike = None,
    *,
    classifier: bool = False,
) -> BaseCrossValidator:
    ...


def train_test_split(
    *arrays,
    test_size: None | Float = None,
    train_size: None | Float = None,
    random_state: RandomState | None | Int = None,
    shuffle: bool = True,
    stratify: None | ArrayLike = None,
) -> list:
    ...



from collections.abc import Hashable

import numpy as np
from pandas.core.frame import DataFrame

from pandas._typing import HashableT

def melt(
    frame: DataFrame,
    id_vars: tuple | list | np.ndarray | None = ...,
    value_vars: tuple | list | np.ndarray | None = ...,
    var_name: str | None = ...,
    value_name: Hashable = ...,
    col_level: int | str | None = ...,
    ignore_index: bool = ...,
) -> DataFrame:
    """
Unpivot a DataFrame from wide to long format, optionally leaving identifiers set.

This function is useful to massage a DataFrame into a format where one
or more columns are identifier variables (`id_vars`), while all other
columns, considered measured variables (`value_vars`), are "unpivoted" to
the row axis, leaving just two non-identifier columns, 'variable' and
'value'.

Parameters
----------
id_vars : tuple, list, or ndarray, optional
    Column(s) to use as identifier variables.
value_vars : tuple, list, or ndarray, optional
    Column(s) to unpivot. If not specified, uses all columns that
    are not set as `id_vars`.
var_name : scalar
    Name to use for the 'variable' column. If None it uses
    ``frame.columns.name`` or 'variable'.
value_name : scalar, default 'value'
    Name to use for the 'value' column.
col_level : int or str, optional
    If columns are a MultiIndex then use this level to melt.
ignore_index : bool, default True
    If True, original index is ignored. If False, the original index is retained.
    Index labels will be repeated as necessary.

    .. versionadded:: 1.1.0

Returns
-------
DataFrame
    Unpivoted DataFrame.

See Also
--------
DataFrame.melt : Identical method.
pivot_table : Create a spreadsheet-style pivot table as a DataFrame.
DataFrame.pivot : Return reshaped DataFrame organized
    by given index / column values.
DataFrame.explode : Explode a DataFrame from list-like
        columns to long format.

Notes
-----
Reference :ref:`the user guide <reshaping.melt>` for more examples.

Examples
--------
>>> df = pd.DataFrame({'A': {0: 'a', 1: 'b', 2: 'c'},
...                    'B': {0: 1, 1: 3, 2: 5},
...                    'C': {0: 2, 1: 4, 2: 6}})
>>> df
   A  B  C
0  a  1  2
1  b  3  4
2  c  5  6

>>> pd.melt(df, id_vars=['A'], value_vars=['B'])
   A variable  value
0  a        B      1
1  b        B      3
2  c        B      5

>>> pd.melt(df, id_vars=['A'], value_vars=['B', 'C'])
   A variable  value
0  a        B      1
1  b        B      3
2  c        B      5
3  a        C      2
4  b        C      4
5  c        C      6

The names of 'variable' and 'value' columns can be customized:

>>> pd.melt(df, id_vars=['A'], value_vars=['B'],
...         var_name='myVarname', value_name='myValname')
   A myVarname  myValname
0  a         B          1
1  b         B          3
2  c         B          5

Original index values can be kept around:

>>> pd.melt(df, id_vars=['A'], value_vars=['B', 'C'], ignore_index=False)
   A variable  value
0  a        B      1
1  b        B      3
2  c        B      5
0  a        C      2
1  b        C      4
2  c        C      6

If you have multi-index columns:

>>> df.columns = [list('ABC'), list('DEF')]
>>> df
   A  B  C
   D  E  F
0  a  1  2
1  b  3  4
2  c  5  6

>>> pd.melt(df, col_level=0, id_vars=['A'], value_vars=['B'])
   A variable  value
0  a        B      1
1  b        B      3
2  c        B      5

>>> pd.melt(df, id_vars=[('A', 'D')], value_vars=[('B', 'E')])
  (A, D) variable_0 variable_1  value
0      a          B          E      1
1      b          B          E      3
2      c          B          E      5
    """
    pass
def lreshape(
    data: DataFrame, groups: dict[HashableT, list[HashableT]], dropna: bool = ...
) -> DataFrame: ...
def wide_to_long(
    df: DataFrame,
    stubnames: str | list[str],
    i: str | list[str],
    j: str,
    sep: str = ...,
    suffix: str = ...,
) -> DataFrame: ...



from typing import Mapping
from networkx import NetworkXError

from ..classes.graph import Graph
from ..utils import not_implemented_for

__all__ = ["reciprocity", "overall_reciprocity"]

def reciprocity(G: Graph, nodes=None) -> Mapping: ...
def overall_reciprocity(G: Graph): ...



from ._hist_gradient_boosting.gradient_boosting import (
    HistGradientBoostingRegressor as HistGradientBoostingRegressor,
    HistGradientBoostingClassifier as HistGradientBoostingClassifier,
)
from ._voting import (
    VotingClassifier as VotingClassifier,
    VotingRegressor as VotingRegressor,
)
from ._iforest import IsolationForest as IsolationForest
from ._base import BaseEnsemble as BaseEnsemble
from ._gb import (
    GradientBoostingClassifier as GradientBoostingClassifier,
    GradientBoostingRegressor as GradientBoostingRegressor,
)
from ._stacking import (
    StackingClassifier as StackingClassifier,
    StackingRegressor as StackingRegressor,
)
from ._bagging import (
    BaggingClassifier as BaggingClassifier,
    BaggingRegressor as BaggingRegressor,
)
from ._forest import (
    RandomForestClassifier as RandomForestClassifier,
    RandomForestRegressor as RandomForestRegressor,
    RandomTreesEmbedding as RandomTreesEmbedding,
    ExtraTreesClassifier as ExtraTreesClassifier,
    ExtraTreesRegressor as ExtraTreesRegressor,
)
from ._weight_boosting import (
    AdaBoostClassifier as AdaBoostClassifier,
    AdaBoostRegressor as AdaBoostRegressor,
)

__all__ = [
    "BaseEnsemble",
    "RandomForestClassifier",
    "RandomForestRegressor",
    "RandomTreesEmbedding",
    "ExtraTreesClassifier",
    "ExtraTreesRegressor",
    "BaggingClassifier",
    "BaggingRegressor",
    "IsolationForest",
    "GradientBoostingClassifier",
    "GradientBoostingRegressor",
    "AdaBoostClassifier",
    "AdaBoostRegressor",
    "VotingClassifier",
    "VotingRegressor",
    "StackingClassifier",
    "StackingRegressor",
    "HistGradientBoostingClassifier",
    "HistGradientBoostingRegressor",
]



from numpy.typing import ArrayLike
from itertools import chain
from ...classes.graph import Graph
from ...utils import not_implemented_for, pairwise

__all__ = ["metric_closure", "steiner_tree"]

def metric_closure(G: Graph, weight="weight"): ...
def steiner_tree(G: Graph, terminal_nodes: ArrayLike, weight="weight"): ...



from typing import Literal
from numpy.random import RandomState
from scipy.special import digamma as digamma
from ..utils.validation import check_array as check_array, check_X_y as check_X_y
from numpy import ndarray
from ..neighbors import NearestNeighbors as NearestNeighbors, KDTree as KDTree
from ..utils.multiclass import (
    check_classification_targets as check_classification_targets,
)
from scipy.sparse import issparse as issparse
from ..metrics.cluster import mutual_info_score as mutual_info_score
from .._typing import MatrixLike, ArrayLike, Int
from ..utils import check_random_state as check_random_state
from ..preprocessing import scale as scale

# Author: Nikolay Mayorov <n59_ru@hotmail.com>
# License: 3-clause BSD

import numpy as np


def mutual_info_regression(
    X: MatrixLike,
    y: ArrayLike,
    *,
    discrete_features: Literal["auto", "auto"] | ArrayLike | bool = "auto",
    n_neighbors: Int = 3,
    copy: bool = True,
    random_state: RandomState | None | Int = None
) -> ndarray:
    ...


def mutual_info_classif(
    X: MatrixLike,
    y: ArrayLike,
    *,
    discrete_features: Literal["auto", "auto"] | ArrayLike | bool = "auto",
    n_neighbors: Int = 3,
    copy: bool = True,
    random_state: RandomState | None | Int = None
) -> ndarray:
    ...



from collections import OrderedDict
from typing import Any, Dict, Iterator, List, Optional

from django.core.serializers import base
from django.core.serializers.base import DeserializedObject
from django.db.models.base import Model

class Serializer(base.Serializer):
    objects: List[Any] = ...
    def get_dump_object(self, obj: Model) -> OrderedDict[Any, Any]: ...

def Deserializer(
    object_list: List[Dict[str, Any]],
    *,
    using: Optional[str] = ...,
    ignorenonexistent: bool = ...,
    **options: Any
) -> Iterator[DeserializedObject]: ...



from collections.abc import Iterable

# Original author: D. Eppstein, UC Irvine, August 12, 2003.
# The original code at http://www.ics.uci.edu/~eppstein/PADS/ is public domain.

__all__ = ["read_leda", "parse_leda"]

from ..classes.graph import Graph
from ..exception import NetworkXError
from ..utils import open_file

def read_leda(path, encoding="UTF-8"): ...
def parse_leda(lines: str | Iterable): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

def _get_edges(adjacency_mat): ...
def _sparse_get_edges(adjacency_mat): ...
def _ndarray_get_edges(adjacency_mat): ...
def _get_directed_edges(adjacency_mat): ...
def _straight_line_vertices(adjacency_mat, node_coords, directed=False): ...
def _rescale_layout(pos, scale=1): ...



from matplotlib.backend_bases import RendererBase
from .figure import Figure
from .axes import Axes
from .transforms import Bbox

def do_constrained_layout(
    fig: Figure,
    h_pad: float,
    w_pad: float,
    hspace: float|None = None,
    wspace: float|None = None,
    rect: tuple = ...
)-> dict: ...
def make_layoutgrids(fig, layoutgrids:dict, rect=...)-> dict: ...
def make_layoutgrids_gs(layoutgrids: dict, gs)-> dict: ...
def check_no_collapsed_axes(layoutgrids, fig: Figure)-> bool: ...
def compress_fixed_aspect(layoutgrids, fig: Figure): ...
def get_margin_from_padding(obj, *, w_pad: float=0, h_pad: float=0, hspace: float=0, wspace: float=0)-> dict: ...
def make_layout_margins(
    layoutgrids: dict, fig: Figure, renderer: RendererBase, *, w_pad: float=0, h_pad: float=0, hspace: float=0, wspace: float=0
)-> dict: ...
def make_margin_suptitles(layoutgrids: dict, fig: Figure, renderer: RendererBase, *, w_pad: float=0, h_pad: float=0)-> None: ...
def match_submerged_margins(layoutgrids: dict, fig: Figure)-> None: ...
def get_cb_parent_spans(cbax)-> tuple[range, range]: ...
def get_pos_and_bbox(ax: Axes, renderer: RendererBase) -> tuple[Bbox, Bbox]: ...
def reposition_axes(
    layoutgrids: dict, fig: Figure, renderer: RendererBase, *, w_pad: float=0, h_pad: float=0, hspace: float=0, wspace: float=0
)-> None: ...
def reposition_colorbar(layoutgrids: dict, cbax: Axes, renderer: RendererBase, *, offset: float|None=None)-> dict: ...
def reset_margins(layoutgrids: dict, fig: Figure)-> None: ...
def colorbar_get_pad(layoutgrids: dict, cax: Axes)-> float: ...



from numpy.typing import ArrayLike

# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.
# -----------------------------------------------------------------------------
# Author: Luke Campagnola
# -----------------------------------------------------------------------------

import struct
import zlib
import numpy as np

def _make_png(data, level=6): ...
def read_png(filename: str) -> np.ndarray: ...
def write_png(filename: str, data: ArrayLike): ...
def imread(filename: str, format: str | None = None) -> np.ndarray: ...
def imsave(filename: str, im: ArrayLike, format: str | None = None): ...
def _check_img_lib(): ...



"""
This type stub file was generated by pyright.
"""

import abc
import functools
from ast import stmt
from pathlib import Path
from typing import Any as AnyType, Callable, List as ListType, Optional, Tuple
from traitlets import Integer, Unicode, observe
from traitlets.config.configurable import SingletonConfigurable
from IPython.core import magic
from IPython.core.debugger import InterruptiblePdb
from IPython.testing.skipdoctest import skip_doctest
from IPython.utils.decorators import undoc
from IPython.utils.process import system

"""Main IPython class."""
sphinxify: Optional[Callable]
def sphinxify(oinfo): # -> (docstring: Unknown) -> Unknown:
    ...

class ProvisionalWarning(DeprecationWarning):
    """
    Warning class for unstable features
    """
    ...


_assign_nodes = ...
_single_targets_nodes = ...
dedent_re = ...
def is_integer_string(s: str):
    """
    Variant of "str.isnumeric()" that allow negative values and other ints.
    """
    ...

@undoc
def softspace(file, newvalue):
    """Copied from code.py, to remove the dependency"""
    ...

@undoc
def no_op(*a, **kw): # -> None:
    ...

class SpaceInInput(Exception):
    ...


class SeparateUnicode(Unicode):
    r"""A Unicode subclass to validate separate_in, separate_out, etc.

    This is a Unicode based trait that converts '0'->'' and ``'\\n'->'\n'``.
    """
    def validate(self, obj, value): # -> None:
        ...
    


@undoc
class DummyMod:
    """A dummy module used for IPython's interactive module when
    a namespace must be assigned to the module's __dict__."""
    __spec__ = ...


class ExecutionInfo:
    """The arguments used for a call to :meth:`InteractiveShell.run_cell`

    Stores information about what is going to happen.
    """
    raw_cell = ...
    store_history = ...
    silent = ...
    shell_futures = ...
    cell_id = ...
    def __init__(self, raw_cell, store_history, silent, shell_futures, cell_id) -> None:
        ...
    
    def __repr__(self):
        ...
    


class ExecutionResult:
    """The result of a call to :meth:`InteractiveShell.run_cell`

    Stores information about what took place.
    """
    execution_count = ...
    error_before_exec = ...
    error_in_exec: Optional[BaseException] = ...
    info = ...
    result = ...
    def __init__(self, info) -> None:
        ...
    
    @property
    def success(self):
        ...
    
    def raise_error(self): # -> None:
        """Reraises error if `success` is `False`, otherwise does nothing"""
        ...
    
    def __repr__(self):
        ...
    


class InteractiveShell(SingletonConfigurable):
    """An enhanced, interactive shell for Python."""
    _instance = ...
    ast_transformers = ...
    autocall = ...
    autoindent = ...
    autoawait = ...
    loop_runner_map = ...
    loop_runner = ...
    automagic = ...
    banner1 = ...
    banner2 = ...
    cache_size = Integer(1000, help="""
        Set the size of the output cache.  The default is 1000, you can
        change it permanently in your config file.  Setting it to 0 completely
        disables the caching system, and the minimum value accepted is 3 (if
        you provide a value less than 3, it is reset to 0 and a warning is
        issued).  This limit is defined because otherwise you'll spend more
        time re-flushing a too small cache than working
        """).tag(config=True)
    color_info = ...
    colors = ...
    debug = ...
    disable_failing_post_execute = ...
    display_formatter = ...
    displayhook_class = ...
    display_pub_class = ...
    compiler_class = ...
    inspector_class = ...
    sphinxify_docstring = ...
    enable_html_pager = ...
    data_pub_class = ...
    exit_now = ...
    exiter = ...
    execution_count = Integer(1)
    filename = ...
    ipython_dir = ...
    input_transformer_manager = ...
    @property
    def input_transformers_cleanup(self):
        ...
    
    input_transformers_post = ...
    @property
    def input_splitter(self): # -> Instance:
        """Make this available for backward compatibility (pre-7.0 release) with existing code.

        For example, ipykernel ipykernel currently uses
        `shell.input_splitter.check_complete`
        """
        ...
    
    logstart = ...
    logfile = ...
    logappend = ...
    object_info_string_level = ...
    pdb = ...
    display_page = ...
    show_rewritten_input = ...
    quiet = ...
    history_length = Integer(10000, help='Total length of command history').tag(config=True)
    history_load_length = Integer(1000, help="""
        The number of saved history entries to be loaded
        into the history buffer at startup.
        """).tag(config=True)
    ast_node_interactivity = ...
    warn_venv = ...
    separate_in = ...
    separate_out = ...
    separate_out2 = ...
    wildcards_case_sensitive = ...
    xmode = ...
    alias_manager = ...
    prefilter_manager = ...
    builtin_trap = ...
    display_trap = ...
    extension_manager = ...
    payload_manager = ...
    history_manager = ...
    magics_manager = ...
    profile_dir = ...
    @property
    def profile(self): # -> None:
        ...
    
    _post_execute = ...
    pylab_gui_select = ...
    last_execution_succeeded = ...
    last_execution_result = ...
    def __init__(self, ipython_dir=..., profile_dir=..., user_module=..., user_ns=..., custom_exceptions=..., **kwargs) -> None:
        ...
    
    def get_ipython(self): # -> Self@InteractiveShell:
        """Return the currently running IPython instance."""
        ...
    
    def set_autoindent(self, value=...): # -> None:
        """Set the autoindent flag.

        If called with no arguments, it acts as a toggle."""
        ...
    
    def set_trio_runner(self, tr): # -> None:
        ...
    
    def init_ipython_dir(self, ipython_dir): # -> None:
        ...
    
    def init_profile_dir(self, profile_dir): # -> None:
        ...
    
    def init_instance_attrs(self): # -> None:
        ...
    
    def init_environment(self): # -> None:
        """Any changes we need to make to the user's environment."""
        ...
    
    def init_encoding(self): # -> None:
        ...
    
    @observe('colors')
    def init_syntax_highlighting(self, changes=...): # -> None:
        ...
    
    def refresh_style(self): # -> None:
        ...
    
    def init_pushd_popd_magic(self): # -> None:
        ...
    
    def init_logger(self): # -> None:
        ...
    
    def init_logstart(self): # -> None:
        """Initialize logging in case it was requested at the command line.
        """
        ...
    
    def init_builtins(self): # -> None:
        ...
    
    @observe('colors')
    def init_inspector(self, changes=...): # -> None:
        ...
    
    def init_io(self): # -> None:
        ...
    
    def init_prompts(self): # -> None:
        ...
    
    def init_display_formatter(self): # -> None:
        ...
    
    def init_display_pub(self): # -> None:
        ...
    
    def init_data_pub(self): # -> None:
        ...
    
    def init_displayhook(self): # -> None:
        ...
    
    @staticmethod
    def get_path_links(p: Path):
        """Gets path links including all symlinks

        Examples
        --------
        In [1]: from IPython.core.interactiveshell import InteractiveShell

        In [2]: import sys, pathlib

        In [3]: paths = InteractiveShell.get_path_links(pathlib.Path(sys.executable))

        In [4]: len(paths) == len(set(paths))
        Out[4]: True

        In [5]: bool(paths)
        Out[5]: True
        """
        ...
    
    def init_virtualenv(self): # -> None:
        """Add the current virtualenv to sys.path so the user can import modules from it.
        This isn't perfect: it doesn't use the Python interpreter with which the
        virtualenv was built, and it ignores the --no-site-packages option. A
        warning will appear suggesting the user installs IPython in the
        virtualenv, but for many cases, it probably works well enough.

        Adapted from code snippets online.

        http://blog.ufsoft.org/2009/1/29/ipython-and-virtualenv
        """
        ...
    
    def save_sys_module_state(self): # -> None:
        """Save the state of hooks in the sys module.

        This has to be called after self.user_module is created.
        """
        ...
    
    def restore_sys_module_state(self): # -> None:
        """Restore the state of the sys module."""
        ...
    
    @property
    def banner(self): # -> Unicode:
        ...
    
    def show_banner(self, banner=...): # -> None:
        ...
    
    def init_hooks(self): # -> None:
        ...
    
    def set_hook(self, name, hook, priority=..., str_key=..., re_key=...): # -> None:
        """set_hook(name,hook) -> sets an internal IPython hook.

        IPython exposes some of its internal API as user-modifiable hooks.  By
        adding your function to one of these hooks, you can modify IPython's
        behavior to call at runtime your own routines."""
        ...
    
    def init_events(self): # -> None:
        ...
    
    def register_post_execute(self, func):
        """DEPRECATED: Use ip.events.register('post_run_cell', func)

        Register a function for calling after code execution.
        """
        ...
    
    def new_main_mod(self, filename, modname):
        """Return a new 'main' module object for user code execution.

        ``filename`` should be the path of the script which will be run in the
        module. Requests with the same filename will get the same module, with
        its namespace cleared.

        ``modname`` should be the module name - normally either '__main__' or
        the basename of the file without the extension.

        When scripts are executed via %run, we must keep a reference to their
        __main__ module around so that Python doesn't
        clear it, rendering references to module globals useless.

        This method keeps said reference in a private dict, keyed by the
        absolute path of the script. This way, for multiple executions of the
        same script we only keep one copy of the namespace (the last one),
        thus preventing memory leaks from old references while allowing the
        objects from the last execution to be accessible.
        """
        ...
    
    def clear_main_mod_cache(self): # -> None:
        """Clear the cache of main modules.

        Mainly for use by utilities like %reset.

        Examples
        --------
        In [15]: import IPython

        In [16]: m = _ip.new_main_mod(IPython.__file__, 'IPython')

        In [17]: len(_ip._main_mod_cache) > 0
        Out[17]: True

        In [18]: _ip.clear_main_mod_cache()

        In [19]: len(_ip._main_mod_cache) == 0
        Out[19]: True
        """
        ...
    
    def init_pdb(self): # -> None:
        ...
    
    call_pdb = ...
    def debugger(self, force=...): # -> None:
        """Call the pdb debugger.

        Keywords:

          - force(False): by default, this routine checks the instance call_pdb
            flag and does not actually invoke the debugger if the flag is false.
            The 'force' option forces the debugger to activate even if the flag
            is false.
        """
        ...
    
    default_user_namespaces = ...
    def init_create_namespaces(self, user_module=..., user_ns=...): # -> None:
        ...
    
    @property
    def user_global_ns(self):
        ...
    
    def prepare_user_module(self, user_module=..., user_ns=...):
        """Prepare the module and namespace in which user code will be run.

        When IPython is started normally, both parameters are None: a new module
        is created automatically, and its __dict__ used as the namespace.

        If only user_module is provided, its __dict__ is used as the namespace.
        If only user_ns is provided, a dummy module is created, and user_ns
        becomes the global namespace. If both are provided (as they may be
        when embedding), user_ns is the local namespace, and user_module
        provides the global namespace.

        Parameters
        ----------
        user_module : module, optional
            The current user module in which IPython is being run. If None,
            a clean module will be created.
        user_ns : dict, optional
            A namespace in which to run interactive commands.

        Returns
        -------
        A tuple of user_module and user_ns, each properly initialised.
        """
        ...
    
    def init_sys_modules(self): # -> None:
        ...
    
    def init_user_ns(self): # -> None:
        """Initialize all user-visible namespaces to their minimum defaults.

        Certain history lists are also initialized here, as they effectively
        act as user namespaces.

        Notes
        -----
        All data structures here are only filled in, they are NOT reset by this
        method.  If they were not empty before, data will simply be added to
        them.
        """
        ...
    
    @property
    def all_ns_refs(self):
        """Get a list of references to all the namespace dictionaries in which
        IPython might store a user-created object.

        Note that this does not include the displayhook, which also caches
        objects from the output."""
        ...
    
    def reset(self, new_session=..., aggressive=...): # -> None:
        """Clear all internal namespaces, and attempt to release references to
        user objects.

        If new_session is True, a new history session will be opened.
        """
        ...
    
    def del_var(self, varname, by_name=...): # -> None:
        """Delete a variable from the various namespaces, so that, as
        far as possible, we're not keeping any hidden references to it.

        Parameters
        ----------
        varname : str
            The name of the variable to delete.
        by_name : bool
            If True, delete variables with the given name in each
            namespace. If False (default), find the variable in the user
            namespace, and delete references to it.
        """
        ...
    
    def reset_selective(self, regex=...): # -> None:
        """Clear selective variables from internal namespaces based on a
        specified regular expression.

        Parameters
        ----------
        regex : string or compiled pattern, optional
            A regular expression pattern that will be used in searching
            variable names in the users namespaces.
        """
        ...
    
    def push(self, variables, interactive=...): # -> None:
        """Inject a group of variables into the IPython user namespace.

        Parameters
        ----------
        variables : dict, str or list/tuple of str
            The variables to inject into the user's namespace.  If a dict, a
            simple update is done.  If a str, the string is assumed to have
            variable names separated by spaces.  A list/tuple of str can also
            be used to give the variable names.  If just the variable names are
            give (list/tuple/str) then the variable values looked up in the
            callers frame.
        interactive : bool
            If True (default), the variables will be listed with the ``who``
            magic.
        """
        ...
    
    def drop_by_id(self, variables): # -> None:
        """Remove a dict of variables from the user namespace, if they are the
        same as the values in the dictionary.

        This is intended for use by extensions: variables that they've added can
        be taken back out if they are unloaded, without removing any that the
        user has overwritten.

        Parameters
        ----------
        variables : dict
            A dictionary mapping object names (as strings) to the objects.
        """
        ...
    
    def object_inspect(self, oname, detail_level=...): # -> Any:
        """Get object info about oname"""
        ...
    
    def object_inspect_text(self, oname, detail_level=...): # -> Any:
        """Get object info as formatted text"""
        ...
    
    def object_inspect_mime(self, oname, detail_level=..., omit_sections=...): # -> Any:
        """Get object info as a mimebundle of formatted representations.

        A mimebundle is a dictionary, keyed by mime-type.
        It must always have the key `'text/plain'`.
        """
        ...
    
    def init_history(self): # -> None:
        """Sets up the command history, and starts regular autosaves."""
        ...
    
    debugger_cls = InterruptiblePdb
    def init_traceback_handlers(self, custom_exceptions): # -> None:
        ...
    
    def set_custom_exc(self, exc_tuple, handler): # -> None:
        """set_custom_exc(exc_tuple, handler)

        Set a custom exception handler, which will be called if any of the
        exceptions in exc_tuple occur in the mainloop (specifically, in the
        run_code() method).

        Parameters
        ----------
        exc_tuple : tuple of exception classes
            A *tuple* of exception classes, for which to call the defined
            handler.  It is very important that you use a tuple, and NOT A
            LIST here, because of the way Python's except statement works.  If
            you only want to trap a single exception, use a singleton tuple::

                exc_tuple == (MyCustomException,)

        handler : callable
            handler must have the following signature::

                def my_handler(self, etype, value, tb, tb_offset=None):
                    ...
                    return structured_traceback

            Your handler must return a structured traceback (a list of strings),
            or None.

            This will be made into an instance method (via types.MethodType)
            of IPython itself, and it will be called if any of the exceptions
            listed in the exc_tuple are caught. If the handler is None, an
            internal basic one is used, which just prints basic info.

            To protect IPython from crashes, if your handler ever raises an
            exception or returns an invalid result, it will be immediately
            disabled.

        Notes
        -----
        WARNING: by putting in your own exception handler into IPython's main
        execution loop, you run a very good chance of nasty crashes.  This
        facility should only be used if you really know what you are doing.
        """
        ...
    
    def excepthook(self, etype, value, tb): # -> None:
        """One more defense for GUI apps that call sys.excepthook.

        GUI frameworks like wxPython trap exceptions and call
        sys.excepthook themselves.  I guess this is a feature that
        enables them to keep running after exceptions that would
        otherwise kill their mainloop. This is a bother for IPython
        which expects to catch all of the program exceptions with a try:
        except: statement.

        Normally, IPython sets sys.excepthook to a CrashHandler instance, so if
        any app directly invokes sys.excepthook, it will look to the user like
        IPython crashed.  In order to work around this, we can disable the
        CrashHandler and replace it with this excepthook instead, which prints a
        regular traceback using our InteractiveTB.  In this fashion, apps which
        call sys.excepthook will generate a regular-looking exception from
        IPython, and the CrashHandler will only be triggered by real IPython
        crashes.

        This hook should be used sparingly, only in places which are not likely
        to be true IPython errors.
        """
        ...
    
    def show_usage_error(self, exc): # -> None:
        """Show a short message for UsageErrors

        These are special exceptions that shouldn't show a traceback.
        """
        ...
    
    def get_exception_only(self, exc_tuple=...):
        """
        Return as a string (ending with a newline) the exception that
        just occurred, without any traceback.
        """
        ...
    
    def showtraceback(self, exc_tuple=..., filename=..., tb_offset=..., exception_only=..., running_compiled_code=...):
        """Display the exception that just occurred.

        If nothing is known about the exception, this is the method which
        should be used throughout the code for presenting user tracebacks,
        rather than directly invoking the InteractiveTB object.

        A specific showsyntaxerror() also exists, but this method can take
        care of calling it if needed, so unless you are explicitly catching a
        SyntaxError exception, don't try to analyze the stack manually and
        simply call this method."""
        ...
    
    def showsyntaxerror(self, filename=..., running_compiled_code=...): # -> None:
        """Display the syntax error that just occurred.

        This doesn't display a stack trace because there isn't one.

        If a filename is given, it is stuffed in the exception instead
        of what was there before (because Python's parser always uses
        "<string>" when reading from a string).

        If the syntax error occurred when running a compiled code (i.e. running_compile_code=True),
        longer stack trace will be displayed.
        """
        ...
    
    def showindentationerror(self): # -> None:
        """Called by _run_cell when there's an IndentationError in code entered
        at the prompt.

        This is overridden in TerminalInteractiveShell to show a message about
        the %paste magic."""
        ...
    
    @skip_doctest
    def set_next_input(self, s, replace=...): # -> None:
        """ Sets the 'default' input string for the next command line.

        Example::

            In [1]: _ip.set_next_input("Hello Word")
            In [2]: Hello Word_  # cursor is here
        """
        ...
    
    def init_completer(self): # -> None:
        """Initialize the completion machinery.

        This creates completion machinery that can be used by client code,
        either interactively in-process (typically triggered by the readline
        library), programmatically (such as in test suites) or out-of-process
        (typically over the network by remote frontends).
        """
        ...
    
    @skip_doctest
    def complete(self, text, line=..., cursor_pos=...):
        """Return the completed text and a list of completions.

        Parameters
        ----------
        text : string
            A string of text to be completed on.  It can be given as empty and
            instead a line/position pair are given.  In this case, the
            completer itself will split the line like readline does.
        line : string, optional
            The complete line that text is part of.
        cursor_pos : int, optional
            The position of the cursor on the input line.

        Returns
        -------
        text : string
            The actual text that was completed.
        matches : list
            A sorted list with all possible completions.

        Notes
        -----
        The optional arguments allow the completion to take more context into
        account, and are part of the low-level completion API.

        This is a wrapper around the completion mechanism, similar to what
        readline does at the command line when the TAB key is hit.  By
        exposing it as a method, it can be used by other non-readline
        environments (such as GUIs) for text completion.

        Examples
        --------
        In [1]: x = 'hello'

        In [2]: _ip.complete('x.l')
        Out[2]: ('x.l', ['x.ljust', 'x.lower', 'x.lstrip'])
        """
        ...
    
    def set_custom_completer(self, completer, pos=...) -> None:
        """Adds a new custom completer function.

        The position argument (defaults to 0) is the index in the completers
        list where you want the completer to be inserted.

        `completer` should have the following signature::

            def completion(self: Completer, text: string) -> List[str]:
                raise NotImplementedError

        It will be bound to the current Completer instance and pass some text
        and return a list with current completions to suggest to the user.
        """
        ...
    
    def set_completer_frame(self, frame=...): # -> None:
        """Set the frame of the completer."""
        ...
    
    def init_magics(self): # -> None:
        ...
    
    @functools.wraps(magic.MagicsManager.register_function)
    def register_magic_function(self, func, magic_kind=..., magic_name=...): # -> None:
        ...
    
    def run_line_magic(self, magic_name: str, line, _stack_depth=...): # -> None:
        """Execute the given line magic.

        Parameters
        ----------
        magic_name : str
            Name of the desired magic function, without '%' prefix.
        line : str
            The rest of the input line as a single string.
        _stack_depth : int
            If run_line_magic() is called from magic() then _stack_depth=2.
            This is added to ensure backward compatibility for use of 'get_ipython().magic()'
        """
        ...
    
    def get_local_scope(self, stack_depth):
        """Get local scope at given stack depth.

        Parameters
        ----------
        stack_depth : int
            Depth relative to calling frame
        """
        ...
    
    def run_cell_magic(self, magic_name, line, cell): # -> None:
        """Execute the given cell magic.

        Parameters
        ----------
        magic_name : str
            Name of the desired magic function, without '%' prefix.
        line : str
            The rest of the first input line as a single string.
        cell : str
            The body of the cell as a (possibly multiline) string.
        """
        ...
    
    def find_line_magic(self, magic_name):
        """Find and return a line magic by name.

        Returns None if the magic isn't found."""
        ...
    
    def find_cell_magic(self, magic_name):
        """Find and return a cell magic by name.

        Returns None if the magic isn't found."""
        ...
    
    def find_magic(self, magic_name, magic_kind=...):
        """Find and return a magic of the given type by name.

        Returns None if the magic isn't found."""
        ...
    
    def magic(self, arg_s): # -> None:
        """
        DEPRECATED

        Deprecated since IPython 0.13 (warning added in
        8.1), use run_line_magic(magic_name, parameter_s).

        Call a magic function by name.

        Input: a string containing the name of the magic function to call and
        any additional arguments to be passed to the magic.

        magic('name -opt foo bar') is equivalent to typing at the ipython
        prompt:

        In[1]: %name -opt foo bar

        To call a magic without arguments, simply use magic('name').

        This provides a proper Python function to call IPython's magics in any
        valid Python code you can type at the interpreter, including loops and
        compound statements.
        """
        ...
    
    def define_macro(self, name, themacro): # -> None:
        """Define a new macro

        Parameters
        ----------
        name : str
            The name of the macro.
        themacro : str or Macro
            The action to do upon invoking the macro.  If a string, a new
            Macro object is created by passing the string to it.
        """
        ...
    
    def system_piped(self, cmd): # -> None:
        """Call the given cmd in a subprocess, piping stdout/err

        Parameters
        ----------
        cmd : str
            Command to execute (can not end in '&', as background processes are
            not supported.  Should not be a command that expects input
            other than simple text.
        """
        ...
    
    def system_raw(self, cmd): # -> None:
        """Call the given cmd in a subprocess using os.system on Windows or
        subprocess.call using the system shell on other platforms.

        Parameters
        ----------
        cmd : str
            Command to execute.
        """
        ...
    
    system = ...
    def getoutput(self, cmd, split=..., depth=...): # -> SList | LSString:
        """Get output (possibly including stderr) from a subprocess.

        Parameters
        ----------
        cmd : str
            Command to execute (can not end in '&', as background processes are
            not supported.
        split : bool, optional
            If True, split the output into an IPython SList.  Otherwise, an
            IPython LSString is returned.  These are objects similar to normal
            lists and strings, with a few convenience attributes for easier
            manipulation of line-based output.  You can use '?' on them for
            details.
        depth : int, optional
            How many frames above the caller are the local variables which should
            be expanded in the command string? The default (0) assumes that the
            expansion variables are in the stack frame calling this function.
        """
        ...
    
    def init_alias(self): # -> None:
        ...
    
    def init_extension_manager(self): # -> None:
        ...
    
    def init_payload(self): # -> None:
        ...
    
    def init_prefilter(self): # -> None:
        ...
    
    def auto_rewrite_input(self, cmd): # -> None:
        """Print to the screen the rewritten form of the user's command.

        This shows visual feedback by rewriting input lines that cause
        automatic calling to kick in, like::

          /f x

        into::

          ------> f(x)

        after the user's input prompt.  This helps the user understand that the
        input line was transformed automatically by IPython.
        """
        ...
    
    def user_expressions(self, expressions):
        """Evaluate a dict of expressions in the user's namespace.

        Parameters
        ----------
        expressions : dict
            A dict with string keys and string values.  The expression values
            should be valid Python expressions, each of which will be evaluated
            in the user namespace.

        Returns
        -------
        A dict, keyed like the input expressions dict, with the rich mime-typed
        display_data of each value.
        """
        ...
    
    def ex(self, cmd): # -> None:
        """Execute a normal python statement in user namespace."""
        ...
    
    def ev(self, expr):
        """Evaluate python expression expr in user namespace.

        Returns the result of evaluation
        """
        ...
    
    def safe_execfile(self, fname, *where, exit_ignore=..., raise_exceptions=..., shell_futures=...): # -> None:
        """A safe version of the builtin execfile().

        This version will never throw an exception, but instead print
        helpful error messages to the screen.  This only works on pure
        Python files with the .py extension.

        Parameters
        ----------
        fname : string
            The name of the file to be executed.
        *where : tuple
            One or two namespaces, passed to execfile() as (globals,locals).
            If only one is given, it is passed as both.
        exit_ignore : bool (False)
            If True, then silence SystemExit for non-zero status (it is always
            silenced for zero status, as it is so common).
        raise_exceptions : bool (False)
            If True raise exceptions everywhere. Meant for testing.
        shell_futures : bool (False)
            If True, the code will share future statements with the interactive
            shell. It will both be affected by previous __future__ imports, and
            any __future__ imports in the code will affect the shell. If False,
            __future__ imports are not shared in either direction.

        """
        ...
    
    def safe_execfile_ipy(self, fname, shell_futures=..., raise_exceptions=...): # -> None:
        """Like safe_execfile, but for .ipy or .ipynb files with IPython syntax.

        Parameters
        ----------
        fname : str
            The name of the file to execute.  The filename must have a
            .ipy or .ipynb extension.
        shell_futures : bool (False)
            If True, the code will share future statements with the interactive
            shell. It will both be affected by previous __future__ imports, and
            any __future__ imports in the code will affect the shell. If False,
            __future__ imports are not shared in either direction.
        raise_exceptions : bool (False)
            If True raise exceptions everywhere.  Meant for testing.
        """
        ...
    
    def safe_run_module(self, mod_name, where): # -> None:
        """A safe version of runpy.run_module().

        This version will never throw an exception, but instead print
        helpful error messages to the screen.

        `SystemExit` exceptions with status code 0 or None are ignored.

        Parameters
        ----------
        mod_name : string
            The name of the module to be executed.
        where : dict
            The globals namespace.
        """
        ...
    
    def run_cell(self, raw_cell, store_history=..., silent=..., shell_futures=..., cell_id=...): # -> ExecutionResult:
        """Run a complete IPython cell.

        Parameters
        ----------
        raw_cell : str
            The code (including IPython code such as %magic functions) to run.
        store_history : bool
            If True, the raw and translated cell will be stored in IPython's
            history. For user code calling back into IPython's machinery, this
            should be set to False.
        silent : bool
            If True, avoid side-effects, such as implicit displayhooks and
            and logging.  silent=True forces store_history=False.
        shell_futures : bool
            If True, the code will share future statements with the interactive
            shell. It will both be affected by previous __future__ imports, and
            any __future__ imports in the code will affect the shell. If False,
            __future__ imports are not shared in either direction.

        Returns
        -------
        result : :class:`ExecutionResult`
        """
        ...
    
    def should_run_async(self, raw_cell: str, *, transformed_cell=..., preprocessing_exc_tuple=...) -> bool:
        """Return whether a cell should be run asynchronously via a coroutine runner

        Parameters
        ----------
        raw_cell : str
            The code to be executed

        Returns
        -------
        result: bool
            Whether the code needs to be run with a coroutine runner or not
        .. versionadded:: 7.0
        """
        ...
    
    async def run_cell_async(self, raw_cell: str, store_history=..., silent=..., shell_futures=..., *, transformed_cell: Optional[str] = ..., preprocessing_exc_tuple: Optional[AnyType] = ..., cell_id=...) -> ExecutionResult:
        """Run a complete IPython cell asynchronously.

        Parameters
        ----------
        raw_cell : str
          The code (including IPython code such as %magic functions) to run.
        store_history : bool
          If True, the raw and translated cell will be stored in IPython's
          history. For user code calling back into IPython's machinery, this
          should be set to False.
        silent : bool
          If True, avoid side-effects, such as implicit displayhooks and
          and logging.  silent=True forces store_history=False.
        shell_futures : bool
          If True, the code will share future statements with the interactive
          shell. It will both be affected by previous __future__ imports, and
          any __future__ imports in the code will affect the shell. If False,
          __future__ imports are not shared in either direction.
        transformed_cell: str
          cell that was passed through transformers
        preprocessing_exc_tuple:
          trace if the transformation failed.

        Returns
        -------
        result : :class:`ExecutionResult`

        .. versionadded:: 7.0
        """
        ...
    
    def transform_cell(self, raw_cell):
        """Transform an input cell before parsing it.

        Static transformations, implemented in IPython.core.inputtransformer2,
        deal with things like ``%magic`` and ``!system`` commands.
        These run on all input.
        Dynamic transformations, for things like unescaped magics and the exit
        autocall, depend on the state of the interpreter.
        These only apply to single line inputs.

        These string-based transformations are followed by AST transformations;
        see :meth:`transform_ast`.
        """
        ...
    
    def transform_ast(self, node):
        """Apply the AST transformations from self.ast_transformers

        Parameters
        ----------
        node : ast.Node
            The root node to be transformed. Typically called with the ast.Module
            produced by parsing user input.

        Returns
        -------
        An ast.Node corresponding to the node it was called with. Note that it
        may also modify the passed object, so don't rely on references to the
        original AST.
        """
        ...
    
    async def run_ast_nodes(self, nodelist: ListType[stmt], cell_name: str, interactivity=..., compiler=..., result=...): # -> None:
        """Run a sequence of AST nodes. The execution mode depends on the
        interactivity parameter.

        Parameters
        ----------
        nodelist : list
          A sequence of AST nodes to run.
        cell_name : str
          Will be passed to the compiler as the filename of the cell. Typically
          the value returned by ip.compile.cache(cell).
        interactivity : str
          'all', 'last', 'last_expr' , 'last_expr_or_assign' or 'none',
          specifying which nodes should be run interactively (displaying output
          from expressions). 'last_expr' will run the last node interactively
          only if it is an expression (i.e. expressions in loops or other blocks
          are not displayed) 'last_expr_or_assign' will run the last expression
          or the last assignment. Other values for this parameter will raise a
          ValueError.

        compiler : callable
          A function with the same interface as the built-in compile(), to turn
          the AST nodes into code objects. Default is the built-in compile().
        result : ExecutionResult, optional
          An object to store exceptions that occur during execution.

        Returns
        -------
        True if an exception occurred while running code, False if it finished
        running.
        """
        ...
    
    async def run_code(self, code_obj, result=..., *, async_=...):
        """Execute a code object.

        When an exception occurs, self.showtraceback() is called to display a
        traceback.

        Parameters
        ----------
        code_obj : code object
          A compiled code object, to be executed
        result : ExecutionResult, optional
          An object to store exceptions that occur during execution.
        async_ :  Bool (Experimental)
          Attempt to run top-level asynchronous code in a default loop.

        Returns
        -------
        False : successful execution.
        True : an error occurred.
        """
        ...
    
    runcode = ...
    def check_complete(self, code: str) -> Tuple[str, str]:
        """Return whether a block of code is ready to execute, or should be continued

        Parameters
        ----------
        code : string
            Python input code, which can be multiline.

        Returns
        -------
        status : str
            One of 'complete', 'incomplete', or 'invalid' if source is not a
            prefix of valid code.
        indent : str
            When status is 'incomplete', this is some whitespace to insert on
            the next line of the prompt.
        """
        ...
    
    active_eventloop = ...
    def enable_gui(self, gui=...):
        ...
    
    def enable_matplotlib(self, gui=...):
        """Enable interactive matplotlib and inline figure support.

        This takes the following steps:

        1. select the appropriate eventloop and matplotlib backend
        2. set up matplotlib for interactive use with that backend
        3. configure formatters for inline figure display
        4. enable the selected gui eventloop

        Parameters
        ----------
        gui : optional, string
            If given, dictates the choice of matplotlib GUI backend to use
            (should be one of IPython's supported backends, 'qt', 'osx', 'tk',
            'gtk', 'wx' or 'inline'), otherwise we use the default chosen by
            matplotlib (as dictated by the matplotlib build-time options plus the
            user's matplotlibrc configuration file).  Note that not all backends
            make sense in all contexts, for example a terminal ipython can't
            display figures inline.
        """
        ...
    
    def enable_pylab(self, gui=..., import_all=..., welcome_message=...):
        """Activate pylab support at runtime.

        This turns on support for matplotlib, preloads into the interactive
        namespace all of numpy and pylab, and configures IPython to correctly
        interact with the GUI event loop.  The GUI backend to be used can be
        optionally selected with the optional ``gui`` argument.

        This method only adds preloading the namespace to InteractiveShell.enable_matplotlib.

        Parameters
        ----------
        gui : optional, string
            If given, dictates the choice of matplotlib GUI backend to use
            (should be one of IPython's supported backends, 'qt', 'osx', 'tk',
            'gtk', 'wx' or 'inline'), otherwise we use the default chosen by
            matplotlib (as dictated by the matplotlib build-time options plus the
            user's matplotlibrc configuration file).  Note that not all backends
            make sense in all contexts, for example a terminal ipython can't
            display figures inline.
        import_all : optional, bool, default: True
            Whether to do `from numpy import *` and `from pylab import *`
            in addition to module imports.
        welcome_message : deprecated
            This argument is ignored, no welcome message will be displayed.
        """
        ...
    
    def var_expand(self, cmd, depth=..., formatter=...):
        """Expand python variables in a string.

        The depth argument indicates how many frames above the caller should
        be walked to look for the local namespace where to expand variables.

        The global namespace for expansion is always the user's interactive
        namespace.
        """
        ...
    
    def mktempfile(self, data=..., prefix=...):
        """Make a new tempfile and return its filename.

        This makes a call to tempfile.mkstemp (created in a tempfile.mkdtemp),
        but it registers the created filename internally so ipython cleans it up
        at exit time.

        Optional inputs:

          - data(None): if data is given, it gets written out to the temp file
            immediately, and the file is closed again."""
        ...
    
    def ask_yes_no(self, prompt, default=..., interrupt=...):
        ...
    
    def show_usage(self): # -> None:
        """Show a usage message"""
        ...
    
    def extract_input_lines(self, range_str, raw=...):
        """Return as a string a set of input history slices.

        Parameters
        ----------
        range_str : str
            The set of slices is given as a string, like "~5/6-~4/2 4:8 9",
            since this function is for use by magic functions which get their
            arguments as strings. The number before the / is the session
            number: ~n goes n back from the current session.

            If empty string is given, returns history of current session
            without the last input.

        raw : bool, optional
            By default, the processed input is used.  If this is true, the raw
            input history is used instead.

        Notes
        -----
        Slices can be described with two notations:

        * ``N:M`` -> standard python form, means including items N...(M-1).
        * ``N-M`` -> include items N..M (closed endpoint).
        """
        ...
    
    def find_user_code(self, target, raw=..., py_only=..., skip_encoding_cookie=..., search_ns=...): # -> Any:
        """Get a code string from history, file, url, or a string or macro.

        This is mainly used by magic functions.

        Parameters
        ----------
        target : str
            A string specifying code to retrieve. This will be tried respectively
            as: ranges of input history (see %history for syntax), url,
            corresponding .py file, filename, or an expression evaluating to a
            string or Macro in the user namespace.

            If empty string is given, returns complete history of current
            session, without the last line.

        raw : bool
            If true (default), retrieve raw history. Has no effect on the other
            retrieval mechanisms.

        py_only : bool (default False)
            Only try to fetch python code, do not try alternative methods to decode file
            if unicode fails.

        Returns
        -------
        A string of code.
        ValueError is raised if nothing is found, and TypeError if it evaluates
        to an object of another type. In each case, .args[0] is a printable
        message.
        """
        ...
    
    def atexit_operations(self): # -> None:
        """This will be executed at the time of exit.

        Cleanup operations and saving of persistent data that is done
        unconditionally by IPython should be performed here.

        For things that may depend on startup flags or platform specifics (such
        as having readline or not), register a separate atexit function in the
        code that has the appropriate information, rather than trying to
        clutter
        """
        ...
    
    def cleanup(self): # -> None:
        ...
    
    def switch_doctest_mode(self, mode): # -> None:
        ...
    


class InteractiveShellABC(metaclass=abc.ABCMeta):
    """An abstract base class for InteractiveShell."""
    ...





from typing import Callable

from pandas.core.groupby.base import GroupByMixin as GroupByMixin

class WindowGroupByMixin(GroupByMixin):
    def __init__(self, obj, *args, **kwargs) -> None: ...

def calculate_center_offset(window): ...
def calculate_min_periods(
    window: int,
    min_periods: int | None,
    num_values: int,
    required_min_periods: int,
    floor: int,
) -> int: ...
def zsqrt(x): ...
def prep_binary(arg1, arg2): ...
def get_weighted_roll_func(cfunc: Callable) -> Callable: ...



from numpy.typing import NDArray, ArrayLike
from typing import Literal
import math

import numpy as np

from .._shared import utils
from .._shared.filters import gaussian
from .._shared.utils import convert_to_float
from ..transform import resize

def _smooth(image, sigma, mode, cval, channel_axis): ...
def _check_factor(factor): ...
@utils.deprecate_multichannel_kwarg(multichannel_position=6)
def pyramid_reduce(
    image: NDArray,
    downscale: float = 2,
    sigma: float | None = None,
    order: int = 1,
    mode: Literal["reflect", "constant", "edge", "symmetric", "wrap"] = "reflect",
    cval: float = 0,
    multichannel: bool = False,
    preserve_range: bool = False,
    *,
    channel_axis: int | None = None
) -> ArrayLike: ...
@utils.deprecate_multichannel_kwarg(multichannel_position=6)
def pyramid_expand(
    image: NDArray,
    upscale: float = 2,
    sigma: float | None = None,
    order: int = 1,
    mode: Literal["reflect", "constant", "edge", "symmetric", "wrap"] = "reflect",
    cval: float = 0,
    multichannel: bool = False,
    preserve_range: bool = False,
    *,
    channel_axis: int | None = None
) -> ArrayLike: ...
@utils.deprecate_multichannel_kwarg(multichannel_position=7)
def pyramid_gaussian(
    image: NDArray,
    max_layer: int = ...,
    downscale: float = 2,
    sigma: float | None = None,
    order: int = 1,
    mode: Literal["reflect", "constant", "edge", "symmetric", "wrap"] = "reflect",
    cval: float = 0,
    multichannel: bool = False,
    preserve_range: bool = False,
    *,
    channel_axis: int | None = None
): ...
@utils.deprecate_multichannel_kwarg(multichannel_position=7)
def pyramid_laplacian(
    image: NDArray,
    max_layer: int = ...,
    downscale: float = 2,
    sigma: float | None = None,
    order: int = 1,
    mode: Literal["reflect", "constant", "edge", "symmetric", "wrap"] = "reflect",
    cval: float = 0,
    multichannel: bool = False,
    preserve_range: bool = False,
    *,
    channel_axis: int | None = None
): ...




# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

import numpy as np

from ..visuals import Compound
from ...visuals.mesh import MeshVisual
from ...visuals.transforms import STTransform
from ...visuals.filters import Clipper
from ...util.event import Event
from ...geometry import Rect
from ...color import Color

class Widget(Compound):
    def __init__(
        self,
        pos: tuple[float, float] = ...,
        size: tuple[float, float] = ...,
        border_color=None,
        border_width: float = 1,
        bgcolor=None,
        padding: int = 0,
        margin: int = 0,
        **kwargs,
    ): ...
    @property
    def pos(self): ...
    @pos.setter
    def pos(self, p): ...
    @property
    def size(self): ...
    @size.setter
    def size(self, s): ...
    @property
    def width(self): ...
    @property
    def width_min(self): ...
    @width_min.setter
    def width_min(self, width_min): ...
    @property
    def width_max(self): ...
    @width_max.setter
    def width_max(self, width_max): ...
    @property
    def height(self): ...
    @property
    def height_min(self): ...
    @height_min.setter
    def height_min(self, height_min): ...
    @property
    def height_max(self): ...
    @height_max.setter
    def height_max(self, height_max): ...
    @property
    def rect(self): ...
    @rect.setter
    def rect(self, r): ...
    @property
    def inner_rect(self): ...
    @property
    def stretch(self): ...
    @stretch.setter
    def stretch(self, s): ...
    def _update_layout(self): ...
    def _update_clipper(self): ...
    @property
    def border_color(self): ...
    @border_color.setter
    def border_color(self, b): ...
    @property
    def bgcolor(self): ...
    @bgcolor.setter
    def bgcolor(self, value): ...
    @property
    def margin(self): ...
    @margin.setter
    def margin(self, m): ...
    @property
    def padding(self): ...
    @padding.setter
    def padding(self, p): ...
    def _update_line(self): ...
    def _update_colors(self): ...
    @property
    def picking(self): ...
    @picking.setter
    def picking(self, p): ...
    def _update_visibility(self): ...
    def _update_child_widgets(self): ...
    def add_widget(self, widget: Widget) -> Widget: ...
    def add_grid(self, *args, **kwargs): ...
    def add_view(self, *args, **kwargs): ...
    def remove_widget(self, widget: Widget): ...



from numpy.typing import ArrayLike
from typing import Literal, Callable
import numpy as np
from scipy import ndimage as ndi

from .._shared.utils import _validate_interpolation_order, _fix_ndimage_mode

def profile_line(
    image,
    src,
    dst,
    linewidth: int = 1,
    order=None,
    mode: Literal["constant", "nearest", "reflect", "mirror", "wrap"] = "reflect",
    cval: float = 0.0,
    *,
    reduce_func: Callable = ...
) -> ArrayLike: ...
def _line_profile_coordinates(src, dst, linewidth=1): ...



import numpy as np
from scipy.spatial import distance

def _ensure_spacing(coord, spacing, p_norm, max_out): ...
def ensure_spacing(
    coords,
    spacing: float = 1,
    p_norm: float = ...,
    min_split_size: int = 50,
    max_out: int | None = None,
    *,
    max_split_size: int = 2000
): ...



# -*- coding: utf-8 -*-
# Copyright (c) Vispy Development Team. All Rights Reserved.
# Distributed under the (new) BSD License. See LICENSE.txt for more info.

__all__ = [
    "MeshData",
    "PolygonData",
    "Rect",
    "Triangulation",
    "triangulate",
    "create_arrow",
    "create_box",
    "create_cone",
    "create_cube",
    "create_cylinder",
    "create_grid_mesh",
    "create_plane",
    "create_sphere",
    "resize",
]

from .polygon import PolygonData as PolygonData  # noqa
from .meshdata import MeshData as MeshData  # noqa
from .rect import Rect as Rect  # noqa
from .triangulation import (
    Triangulation as Triangulation,
    triangulate as triangulate,
)  # noqa
from .torusknot import TorusKnot as TorusKnot  # noqa
from .calculations import (
    _calculate_normals as _calculate_normals,
    _fast_cross_3d as _fast_cross_3d,  # noqa
    resize as resize,
)  # noqa
from .generation import (
    create_arrow as create_arrow,
    create_box as create_box,
    create_cone as create_cone,  # noqa
    create_cube as create_cube,
    create_cylinder as create_cylinder,
    create_grid_mesh as create_grid_mesh,  # noqa
    create_plane as create_plane,
    create_sphere as create_sphere,
)  # noqa



# Python: 3.8.2 (tags/v3.8.2:7b3ab59, Feb 25 2020, 23:03:10) [MSC v.1916 64 bit (AMD64)]
# Library: scipy, version: 1.6.2
# Module: scipy._lib.messagestream, version: unspecified
import typing
import builtins as _mod_builtins

class MessageStream(_mod_builtins.object):
    '\n    Capture messages emitted to FILE* streams. Do this by directing them\n    to a temporary file, residing in memory (if possible) or on disk.\n    '
    def __del__(self) -> None:
        ...
    
    def __init__(self, *args, **kwargs) -> None:
        '\n    Capture messages emitted to FILE* streams. Do this by directing them\n    to a temporary file, residing in memory (if possible) or on disk.\n    '
        ...
    
    @classmethod
    def __init_subclass__(cls) -> None:
        'This method is called when a class is subclassed.\n\nThe default implementation does nothing. It may be\noverridden to extend subclasses.\n'
        ...
    
    def __reduce__(self) -> typing.Union[str, typing.Tuple[typing.Any, ...]]:
        ...
    
    def __setstate__(self, state: typing.Any) -> None:
        ...
    
    @classmethod
    def __subclasshook__(cls, subclass: typing.Any) -> bool:
        'Abstract classes can override this to customize issubclass().\n\nThis is invoked early on by abc.ABCMeta.__subclasscheck__().\nIt should return True, False or NotImplemented.  If it returns\nNotImplemented, the normal algorithm is used.  Otherwise, it\noverrides the normal algorithm (and the outcome is cached).\n'
        ...
    
    def clear(self) -> typing.Any:
        ...
    
    def close(self) -> typing.Any:
        ...
    
    def get(self) -> typing.Any:
        ...
    
    def __getattr__(self, name) -> typing.Any:
        ...
    

__doc__: typing.Any
__file__: str
__name__: str
__package__: str
__test__: dict
def __getattr__(name) -> typing.Any:
    ...




"""
This type stub file was generated by pyright.
"""

from typing import List

"""Find the path to LightGBM dynamic library files."""
def find_lib_path() -> List[str]:
    """Find the path to LightGBM library files.

    Returns
    -------
    lib_path: list of str
       List of all found library paths to LightGBM.
    """
    ...





import numpy as np
from .common import X_BINNED_DTYPE_C


def set_bitset_memoryview(bitset: np.ndarray, val: X_BINNED_DTYPE_C) -> None: ...

def set_raw_bitset_from_binned_bitset(raw_bitset: np.ndarray,
                                      binned_bitset: np.ndarray,
                                      categories: np.ndarray) -> None: ...



from ..classes.graph import Graph
from ..utils import not_implemented_for, py_random_state

__all__ = ["random_reference", "lattice_reference", "sigma", "omega"]

@py_random_state(3)
def random_reference(G: Graph, niter=1, connectivity=True, seed=None): ...
@py_random_state(4)
def lattice_reference(G: Graph, niter=5, D=None, connectivity=True, seed=None): ...
@py_random_state(3)
def sigma(G: Graph, niter=100, nrand=10, seed=None) -> float: ...
@py_random_state(3)
def omega(G: Graph, niter=5, nrand=10, seed=None) -> float: ...
